#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <stdio.h>    
#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#include "control.h"
#include "model.h"
#include "fpmath.h"
#include "fp.h" 
#include "string.h"

using namespace std;
#define com_geno	1
#define com_pheno	2
#define com_out		3
#define com_pos		4
#define com_rand 	8
#define com_step 	12
#define com_warm 	13

#define com_version 99
#define com_help    100

#define com_casectrl       1001
#define com_num     3003
#define com_sw    3211
#define com_chains    3212
#define com_tau    3213

#define com_note    3300
#define com_allele_coding 4000
#define com_exclude_maf 4001
#define com_exclude_nopos 4006

#define com_silence 4004
#define com_read_mode 5000
#define com_not_snp 5001

#define com_hmin 6001
#define com_hmax 6002
#define com_pmin 6003
#define com_pmax 6004
#define com_smin 6005
#define com_smax 6006
#define com_nstart 6007

CtrlParam::CtrlParam(void)
{
	pMD = NULL;
	hcom["-g"] = com_geno; 
	hcom["-gen"] = com_geno;
	hcom["-p"] = com_pheno; 
	hcom["-phe"] = com_pheno; 
	hcom["-o"] = com_out;
	hcom["-out"] = com_out;
	hcom["-pos"] = com_pos; // rspos file name;
	hcom["-r"] = com_rand;
	hcom["-R"] = com_rand;
//1* for mcmc related. 	
	hcom["-s"] = com_step;
    hcom["-w"] = com_warm;
//2* for snp calc. 
	hcom["--cc"] = com_casectrl; 
	hcom["-cc"] = com_casectrl; 
//help;     
	hcom["-v"] = com_version; 
	hcom["-ver"] = com_version; 
	hcom["-h"] = com_help;
	hcom["-help"] = com_help; 
	hcom["--help"] = com_help; 
	
	hcom["-sw"] = com_sw;
	hcom["-chain"] = com_chains;
	hcom["-chains"] = com_chains;
	hcom["-tau"] = com_tau;
	hcom["-num"] = com_num;
	hcom["-note"] = com_note;
	hcom["-exclude-maf"] = com_exclude_maf; 
	hcom["--exclude-maf"] = com_exclude_maf; 
	hcom["-exclude-nopos"] = com_exclude_nopos; 
	hcom["--exclude-nopos"] = com_exclude_nopos; 
	hcom["--silence"] = com_silence; 
	hcom["-silence"] = com_silence; 
	hcom["--silent"] = com_silence; 
	hcom["-silent"] = com_silence; 
		 
	hcom["--notsnp"] = com_not_snp; 
	hcom["-notsnp"] = com_not_snp;

	hcom["-hmin"] = com_hmin;
	hcom["-hmax"] = com_hmax;
	hcom["-pmin"] = com_pmin;
	hcom["-pmax"] = com_pmax;
	hcom["-smin"] = com_smin;
	hcom["-smax"] = com_smax;
	hcom["-nstart"] = com_nstart;
} 

CtrlParam::~CtrlParam(void)
{
	;
} 

void CtrlParam::OnHelp(void)
{
	cout << endl; 
	cout << " piMASS version 0.90, released by Y. Guan and M. Stephens" << endl; 
	cout << " " << endl;
	cout << " FILE I/O RELATED OPTIONS." << endl;
	cout << " -g <filename>    " << " repeatable for multiple input files. must pair with -p" << endl;
	cout << " -p <file/0/1/z>  " << " repeatable for multiple input files. must pair with -g" << endl; 
	cout << "    <z/1> pairing genotype individuals have phenotype 0/1" << endl;
	cout << " -pos <file>      " << " repeatable for multiple input files" << endl; 
    cout << " -o <prefix>      " << " prefix of all output files, random seeds will be used by default" << endl;  
	cout << " " << endl; 
	cout << " MCMC RELATED OPTIONS." << endl;
	cout << " -w(warm) <num>   " << " specify burn-in steps" << endl; 
	cout << " -s(step) <num>   " << " specify sampling steps" << endl; 
	cout << " -num <num>         " << " specify recording pace" << endl; 
	cout << " -r <num>         " << " specify random seed, system time by default" << endl; 
	cout << " " << endl; 
	cout << " -cc              " << " calc bf of logit regression on binary phenotype." << endl; 
	cout << " -hmin <num>      " << " specify minimum value for h" << endl; 
	cout << " -hmax <num>      " << " specify maximum value for h" << endl; 
	cout << " -pmin <num>      " << " specify minimum value for p" << endl; 
	cout << " -pmax <num>      " << " specify maximum value for p" << endl; 
	cout << " -smin <num>      " << " specify minimum value for number of SNPs in the model" << endl; 
	cout << " -smax <num>      " << " specify maximum value for number of SNPs in the model" << endl; 
	cout << " -nstart <num>      " << " specify the number of snps in the model to begin with" << endl; 
	cout << " " << endl; 
	cout << " OTHER OPTIONS." << endl;
	cout << " -v(ver)          " << "print version and citation" << endl;
	cout << " -h(help)         " << "print this help" << endl;
	cout << " -exclude-maf <num> " << endl; 
	cout << "          exclude SNPs whose maf < num, default 0.01" << endl; 
	cout << " -exclude-nopos <num> " << endl; 
	cout << "          exclude SNPs that has no position information, 1 = yes (default), 0 = no" << endl; 
	cout << " --silence        " << "no terminal output" <<  endl; 
	cout << " " << endl; 
}

void CtrlParam::PrintHeader(void)
{
	cout << endl; 
	cout << " Developed by Yongtao Guan ytguan.at.gmail.com, all rights to be lefted to GNU GPL." << endl; 
	cout << " References: Guan and Stephens (2010)." << endl;
	cout << endl; 
}

void CtrlParam::BatchRun(int argc, char ** argv)
{
	pMD = new class ModelnData;
                             
	int silence = 0; 
	string note; 
	int gmode = 1; 
	
	for(int i = 1; i < argc; i++) 
	{   
		string str;  
		string opt; 
		if(argv[i][0] != '-') 
			continue;
		
		str.assign(argv[i]);
		opt.assign(str, 0, str.length());

		map<string, int> :: iterator iter; 
		iter = hcom.find(opt); 
		if(iter == hcom.end())
		{
			cout << "-pimass: unknown option: " << opt << endl; 
			safe_exit(); 
		}
		
		switch (hcom[opt]) 
		{			
			case com_not_snp:
				pMD->Setmnotsnp(1); 
				break;
			case com_read_mode:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				gmode = atoi(argv[i+1]); 
				break;
			case com_silence:
				silence = 1; 
				pMD->Setsilence(1);
				break; 
			case com_exclude_maf:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					double exmaf = atof(argv[i+1]); 
					if(exmaf == 0)
						pMD->Setexcludemaf(-100);
					else 
						pMD->Setexcludemaf(exmaf);
				}
			   	break; 
			case com_exclude_nopos:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					int temp = atoi(argv[i+1]); 
					if(temp != 0 && temp != 1) 
					{
						cout << "-pimass: bad argument follow option: " << opt << endl; 
						safe_exit(); 
					}
					pMD->Setexcludenopos(temp);
				}
				break; 
			case com_allele_coding:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				pMD->Setallelecoding(atoi(argv[i+1]));
				break; 
			case com_note:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				note.assign(argv[i+1]);
				break;
			case com_geno:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				str.clear();
				str.assign(argv[i+1]);
				pMD->SetvGin(str);
				break;
			case com_pheno:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				str.assign(argv[i+1]);
				pMD->SetvPin(str);
				break;
			case com_out:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				str.assign(argv[i+1]);
				pMD->SetfnOutput(str);
				break;
			case com_pos:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				str.clear(); 
				str.assign(argv[i+1]);
				pMD->SetvPos(str); 
				break;
			case com_rand: // seed 
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				pMD->SetrandSeed(atoi(argv[i+1]));
				break;
            case com_num:
                if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
                if(!isdigit(argv[i+1][0]))
                {
                    cout << "wrong augument after option." << endl;
                    exit(0);
                }
                pMD->Setnum(atoi(argv[i+1]));
                break;
            case com_sw:
                if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
                if(!isdigit(argv[i+1][0]))
                {
                    cout << "wrong augument after option." << endl;
                    exit(0);
                }
                pMD->Setsw(atof(argv[i+1]));
                break;
            case com_chains:
                if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
                if(!isdigit(argv[i+1][0]))
                {
                    cout << "wrong augument after option." << endl;
                    exit(0);
                }
                pMD->Setchains(atoi(argv[i+1]));
                break;
            case com_tau:
                if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
                if(!isdigit(argv[i+1][0]))
                {
                    cout << "wrong augument after option." << endl;
                    exit(0);
                }
                pMD->Settau(atof(argv[i+1]));
                break;
				
			case com_step:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				pMD->SetnMaxSteps(atoi(argv[i+1]));
				break;

			case com_warm:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				pMD->SetnWarmSteps(atoi(argv[i+1]));
				break;

			case com_hmin:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					double th = atof(argv[i+1]); 
					if(th < 0 || th > 1)
					{
						cout << "wrong argument after option." << endl; 
						exit(0); 
					}
					pMD->set_hmin(th);
				}
				break;

			case com_hmax:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					double th = atof(argv[i+1]); 
					if(th < 0 || th > 1)
					{
						cout << "wrong argument after option." << endl; 
						exit(0); 
					}
					pMD->set_hmax(th);
				}
				break;

			case com_pmin:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					int tp = atoi(argv[i+1]); 
					pMD->set_pmin(tp);
				}
				break;
				
			case com_pmax:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					int tp = atoi(argv[i+1]); 
					pMD->set_pmax(tp);
				}
				break;          

			case com_smin:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					int ts = atoi(argv[i+1]); 
					pMD->set_smin(ts);
				}
				break;
				
			case com_smax:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					int ts = atoi(argv[i+1]); 
					pMD->set_smax(ts);
				}
				break;          

			case com_nstart:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				{
					int tn = atoi(argv[i+1]); 
					pMD->set_nstart(tn);
				}
				break;          
			
			case com_help:
				PrintHeader(); 
				OnHelp();
				safe_exit();
				break;

			case com_version:
				PrintHeader(); 
				safe_exit(); 
				break; 

			case com_casectrl:
				pMD->SetOnCaseCtrl(1); 
				break; 

			default:
				fprintf(stderr,"Bad option %s\n", argv[i]);
				OnHelp();
				exit(0);
				break; 
		}              
	}
    
	streambuf * cout_strbuf(cout.rdbuf()); 
	//save current cout into a streambuf; 
	ostringstream output; 
	if(silence == 1) 
		cout.rdbuf(output.rdbuf()); 
	//redirect cout to a streambuf output; 

	int randseed  = pMD->GetrandSeed();  // default returns -1;
	if (randseed <= 0)
	{
		struct timeval beg;
		struct timezone tz;
		gettimeofday(&beg, &tz);
		randseed = (unsigned) (beg.tv_sec + beg.tv_usec);
//			randseed = (unsigned) time(NULL);
	}
	gsl_rng_set(gsl_r, randseed);

	cout << "## random seed = " << randseed << endl; 
	string prefix(pMD->GetfnOutput()); 
	if(prefix.compare("\0") == 0) {
		char tmp[100];
		memset(tmp, '\0', 100);
		sprintf(tmp, "%d", randseed); 
		prefix.assign(tmp); 
		pMD->SetfnOutput(prefix); 
	}
	//if no random seed provided, use system clock;    
	//if no prefix provided, use randseed; 
	
	pMD->open_log(); 
	(pMD->fplog) << "## " << note << endl;
	(pMD->fplog) << "## COMMAND: "; 
	for (int i = 0; i < argc; i++)
		(pMD->fplog) << argv[i] << " "; 
	(pMD->fplog) << endl; 
	(pMD->fplog) << "## randseed = " << randseed << endl; 
    // write command line and notes into log; 
	
//	for (int i = 0; i < 10000; i++)
//		pMD->fplog << geometric(15, 2000) << " "; 
//    pMD->fplog << endl; 
//	for (int i = 0; i < 15; i++)
//		pMD->fplog << pgeometric(i, 15, 2000) << " "; 
//    pMD->fplog << endl; 
	//grant; 
	int read_dip, read_phe; 
	if(gmode > 0) 
	{
		read_dip = pMD->read_bimbam_genotype_distribution(gmode, 1, 0, -1); 
		read_phe = pMD->read_bimbam_phenotype(1); 
	}
	else 
	{
		cout << "## wrong genotyep files ... "; 
		exit(0); 
	}

	time_t sec_beg, sec_end; 
	sec_beg = time(NULL); 
	if(read_dip && read_phe)
	{
		pMD->mcmc(); 
		pMD->fplog << "## random seed = " << randseed << endl; 
		cout << "## random seed = " << randseed << endl; 
		pMD->close_log(); 
	}
	sec_end = time(NULL); 
	(pMD->fplog) << "## EM seconds used = " << sec_end - sec_beg << endl; 
	
	if(silence == 1)
		cout.rdbuf(cout_strbuf); 
	//restore the original cout; 
	delete pMD; 
}

