#include <stdio.h>
#include <stdlib.h> 
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <algorithm>
#include <vector>
#include <set>
#include <map>
#include "model.h"
#include "fpmath.h"
#include "control.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_permutation.h"
#include "gsl/gsl_permute.h"
#include "gsl/gsl_randist.h"
#include "gsl/gsl_cdf.h"
using namespace std;

#if defined (MPI_ENABLED)
#include "mpi.h"
#endif

#define NA (54321)
#define FIXED_SIGMA (0.04)
#define FIXED_PI (6e-4)

void ModelnData::print_progress_bar(int last, char* str, int p, int total)
{
	if(m_silence == 1) return; 
	int progress = (int) (100.0 * p / total); 
	int barsize = (int) (progress / 2.0); 
	char bar[100];
	memset(bar, '\0', 100); 
	if(!last) {
		for (int i = 0; i < barsize; i++)
			bar[i] = '>'; 
		printf("%s [%-50s] %d%%\r", str, bar, progress); 
		fflush(stdout); 	
	} else {
		for (int i = 0; i < barsize; i++)
			bar[i] = '>'; 
		printf("%s [%-50s] 100%%\n", str, bar); 
	}
}

CaseCtrlData::CaseCtrlData(int row, int col, real * vph) 
{
	ni = row;        //number of indiv;
	np = col;       // dim of beta; 
	mx=gsl_matrix_alloc(ni, np); 
	vy=gsl_vector_alloc(ni); 
	delta = gsl_vector_alloc(np); 
	for (int i = 0; i < ni; i++)
		gsl_vector_set(vy, i, vph[i]); 
}

CaseCtrlData::~CaseCtrlData() 
{
	gsl_matrix_free(mx);
	gsl_vector_free(vy); 
	gsl_vector_free(delta); 
}

ModelnData::ModelnData(void)
{
	m_bufsize = 1e4; 
	m_count1 = m_count2 = m_count3 = m_count4 = 0; 
	m_clock1 = m_clock2 = m_clock3 = m_clock4 = 0; 
	m_chains = 1; 
	m_tau = 1; 
	m_sw = 0.333; 
	m_pos2rank = NULL; 
	m_rank2pos = NULL; 
	snpmgt = NULL; 
	m_not_snp = 0; 
	m_silence = 0; 
	m_exclude_maf = -1e100; 
	m_exclude_miss = 0; 
	m_exclude_nopos = 1; 
	m_num = 0; 
	randSeed = 0;
	nMaxSteps = 0;
	nWarmSteps = 0; 
	nLoci = 0; 
	nPH = 1; 

	fnOutput.assign("\0");
	fnRsPos.assign("\0");
	
	cc = 0; 

	m_minh = 0.001; 
	m_maxh = 0.999; 
	m_pmin = 1; 
	m_pmax = 300; 
	m_minsnp = 0;
	m_maxsnp = 300;
	m_nstart = 100; 


}

ModelnData::~ModelnData()
{
	vGin.clear();
	vPin.clear(); 
	vFileIndiv.clear(); 
	vFilePloid.clear(); 
	vv_phval.clear();
	vv_phdex.clear();
	vsRsnum.clear(); 
	nLoci = 0; 
}

int ModelnData::compatibleQ(pair<char, char> p1, pair<char, char> p2)
{   
	pair<char, char> empty;
	if(p1 == empty || p2 == empty) return 1; 
	map<char, char> cc; 
	cc['A'] = 'T'; cc['T'] = 'A';
	cc['C'] = 'G'; cc['G'] = 'C';
	cc['N'] = 'N'; cc['?'] = '?'; 
	cc['+'] = '-'; cc['-'] = '+'; 

	int ac1 = (p1.first != 'N' ? 1 : 0) + (p1.second != 'N' ? 1 : 0); 
	int ac2 = (p2.first != 'N' ? 1 : 0) + (p2.second != 'N' ? 1 : 0); 
	
	int action;   //0: do nothing; 1:flip the second; -1:error; 
	if(ac1 == 1 && ac2 == 1) 
	{
		if(p1.first == p2.first) 
		{
			action = 0;          //no flip; 
		}
		else 
		{
			action = 1;         //flip second one; 
		}   // don't care about major minor here, will take care of it at 012 stage. 2-gt; 
	}

	else if (ac1 == 2 && ac2 == 2) 
	{
		if(p1.first == p2.first && p1.second == p2.second)  //match;
		{
	        action = 0; 
		}
		else if (p1.first == p2.second && p1.second == p2.first) //match allele type, but not maf; 
		{
			action = 1; 
		}   //should generate a warning; 
		else if (p1.first == cc[p2.first] && p1.second == cc[p2.second]) //flip allele type, maf match; 
		{
			action = 0; 
		}   //should generate a warning; 
		else if (p1.first == cc[p2.second] && p1.second == cc[p2.first]) //flip allele type, maf not match; 
		{
			action = 1; 
		}   //should generate a warning; 
		else
			action = -1; 
	}
	else if (ac1 == 1 && ac2 == 2) 
	{
		if(p1.first == p2.first) //match 
		{
			action = 0; 
		}
		else if (p1.first == p2.second) 
		{
			action = 1; 
		}
		else if (p1.first == cc[p2.first])  //flip
		{
			action = 0; 
		}
		else if (p1.first == cc[p2.second]) 
		{
			action = 1; 
		}
		else 
			action = -1; 
	}
	else if (ac1 == 2 && ac2 == 1) 
	{
		if (p2.first == p1.first) 
		{
			action = 0; 
		}
		else if (p2.first == p1.second) 
		{
			action = 1; 
		}
		else if (cc[p2.first] == p1.first) 
		{
			action = 0; 
		}
		else if (cc[p2.first] == p1.second) 
		{
			action = 1; 
		}               
		else 
			action = -1;  
	}
	else    //one of them is missing; 
	{
		action = -1; 
	}
	return action; 
}

char ModelnData::merge_mmSNP(class mmSNP & ref, class mmSNP & s)
{   
	//return the major as coding; 
	map<char, char> cc; 
	cc['A'] = 'T'; cc['T'] = 'A';
	cc['C'] = 'G'; cc['G'] = 'C';
	cc['N'] = 'N'; cc['?'] = '?'; 
	cc['+'] = '-'; cc['-'] = '+'; 
    
    char coding = 'N';  
	//both mono;   //need to update ref allele type; 
	//both poly;   //match allele type, update allele count; 
	//ref mono, new poly;  //direct match? Yes (update ref allele) : No (flip and update ref allele); flippable? 
	//ref poly, new mono; 
	int ac1 = (ref.A != 'N' ? 1 : 0) + (ref.B != 'N' ? 1 : 0); 
	int ac2 = (s.A != 'N' ? 1 : 0) + (s.B != 'N' ? 1 : 0); 

	if(ac1 == 1 && ac2 == 1) 
	{
		coding = s.A; 
		if(ref.A == s.A) 
		{
			ref.countA += s.countA; 
			ref.countB += s.countB; 
			ref.countQ += s.countQ;   
		}
		else 
		{
			ref.B = s.A; 
			ref.countA += s.countB; 
			ref.countB += s.countA; 
			ref.countQ += s.countQ; 
		}    
		fplog << "## WARNING: monomorphic SNPs ..." << endl; 
	}

	else if (ac1 == 2 && ac2 == 2) 
	{
		if(ref.A == s.A && ref.B == s.B)  //match;
		{
			coding = s.A; 
			ref.countA += s.countA; 
			ref.countB += s.countB; 
			ref.countQ += s.countQ;   
		}
		else if (ref.A == cc[s.A] && ref.B == cc[s.B]) //flip allele type, maf match; 
		{
			coding = s.A; 
			ref.countA += s.countA; 
			ref.countB += s.countB; 
			ref.countQ += s.countQ;   
		}   //should generate a warning; 
		else if (ref.A == s.B && ref.B == s.A) //match allele type, but not maf; 
		{
			coding = s.B;   
			ref.countA += s.countB; 
			ref.countB += s.countA; 
			ref.countQ += s.countQ;   
		}   //should generate a warning; 
		else if (ref.A == cc[s.B] && ref.B == cc[s.A]) //flip allele type, maf not match; 
		{
			coding = s.B; 
			ref.countA += s.countB; 
			ref.countB += s.countA; 
			ref.countQ += s.countQ;   
		}   //should generate a warning; 
		else
			coding = 'X'; 
	}
	else if (ac1 == 1 && ac2 == 2) 
	{
		if(ref.A == s.A) //match 
		{
			coding = s.A; 
			ref.B = s.B;  //update allele type; 
			ref.countA += s.countA; 
			ref.countB += s.countB; 
			ref.countQ += s.countQ;   
		}
		else if (ref.A == s.B) 
		{
			coding = s.B; 
			ref.B = s.A; 
			ref.countA += s.countB; 
			ref.countB += s.countA; 
			ref.countQ += s.countQ;   
		}
		else if (ref.A == cc[s.A])  //flip
		{
			coding = s.A; 
			ref.B = cc[s.B]; 
			ref.countA += s.countA; 
			ref.countB += s.countB; 
			ref.countQ += s.countQ;   
		}
		else if (ref.A == cc[s.B]) 
		{
			coding = s.B; 
			ref.B = cc[s.A]; 
			ref.countA += s.countB; 
			ref.countB += s.countA; 
			ref.countQ += s.countQ;   
		}
		else 
			coding = 'X'; 
	}
	else if (ac1 == 2 && ac2 == 1)      
	{
		if (ref.A == s.A || ref.A == cc[s.A])  //reference allele match the first allele; 
		{
			coding = s.A; 
			ref.countA += s.countA; 
			ref.countB += s.countB; 
			ref.countQ += s.countQ;   
		}
		else if (ref.B == s.A || ref.B ==  cc[s.A])    //fist allele match the second reference allele; 
		{
			coding = s.A;         //these genotypes should be 0 in s; 
			ref.countA += s.countB; 
			ref.countB += s.countA; 
			ref.countQ += s.countQ;   
		}   
		else 
			coding = 'X'; 
	}
	else if(ac1 == 0)    
	{
		coding = s.A; 
		ref.countA += s.countA; 
		ref.countB += s.countB; 
		ref.countQ += s.countQ;   
	}
	else if(ac2 == 0) 
	{
		coding = ref.A; 
	}

	return coding; 
}

int ModelnData::read_rs2pos(string fn, long start_pos, long end_pos, long flank) 
{
	ifstream infile; 
	char delimit[] = ";, \t";
	streambuf * pbuf;
	int snpcount = 0; 

    map<string, long> :: iterator liter; 
    map<string, int> :: iterator iter; 
	
	//first, read in the rs pos file; and put into vRsPos;
	infile.open(fn.c_str(), ios::in); 
	if(!infile.is_open()) 
	{
		return 0; 
	}
	pbuf = infile.rdbuf(); 	
	string line; 
	line.assign(getline(pbuf)); 
	while (!line.empty()) 
	{
		line.append(" 0 "); 
		char * res = strtok((char *)line.c_str(), delimit); 
		string rs(res); 
		res = strtok(NULL, delimit); 
		long pos = atol(res);
		res = strtok(NULL, delimit); 
		int chr = atoi(res);
		
		if (end_pos > 0) 
		{
			if(pos > start_pos - flank && pos < end_pos + flank)
			{
				liter = mapRs2pos.find(rs); 
				if(liter == mapRs2pos.end())
					mapRs2pos.insert(pair<string, long> (rs, (long)pos));
				else 
					liter->second = pos; 
				
				iter = mapRs2chr.find(rs); 
				if(iter == mapRs2chr.end())
					mapRs2chr.insert(pair<string, int> (rs, chr)); 
				else 
					iter->second = chr; 
				snpcount++;
			}
		}
		else 
		{
			mapRs2pos.insert(pair<string, long> (rs, pos));
			mapRs2chr.insert(pair<string, int> (rs, chr)); 
			snpcount++;
		}
		line.assign(getline(pbuf)); 
	}
	infile.close();
	return snpcount; 
}

bool rscomp(pair<string, pair<long, int> > a, pair<string, pair<long, int> > b)
{
	if(a.second.second == b.second.second)
		return (a.second.first < b.second.first); 
	else 
		return(a.second.second < b.second.second); 
}

int ModelnData::read_bimbam_genotype_distribution(int mode, int mcmc, long start_pos, long end_pos)   
	//mode=1: mean geno; mode=2:posterior distribution;
{   
	if(mcmc && (!m_not_snp)) m_exclude_maf = 0; 
	if(mcmc && m_exclude_miss == 1.0) m_exclude_miss = 0.10; 
	if(vGin.size() == 0 || vGin.size() != vPin.size()) 
	{
		fplog << "## pimass: genotype phenotype files not match" << endl; 
		cout << "-pimass: genotype phenotype files not match" << endl; 
		return 0; 
	}
	
	fplog << "## pimass: Number of position files = " << vPos.size() << endl; 
	unsigned readpos = 0; 
	for(unsigned f = 0; f < vPos.size(); f++)
	{
		fnRsPos.assign(vPos.at(f)); 
		readpos += read_rs2pos(fnRsPos, start_pos, end_pos, nGeneFlank); 
	}
	if(readpos == 0 && vGin.size() > 1) 
	{
		fplog << "## Quit: BIMBAM requires SNP position file(s) for multiple genotypes" << endl; 
		cout << "-pimass: requires position file(s) for multiple genotypes" << endl; 
		safe_exit(); 
	}   //initialize mapRs2pos; 
	if(readpos < vPos.size()) 
	{
		fplog << "## WARNING: One of position files failed" << endl; 
		cout << "-pimass: one of position files failed" << endl; 
	}

	fplog << "## pimass: Position files contain " << mapRs2pos.size() << " records." << endl; 
	if(vPos.size() > 0)
		cout << "-pimass: position files contain " << mapRs2pos.size() << " records." << endl; 
	
	{

		//first pass, go over all the files collecting summary for each SNP. 
		//allele type, allele count, etc. treating phased panel as unphased (but get the correct nHap, nDip);  
		//merge SNPs based on summary. generate a hash keyed by rs whose content has reference allele
		//for different files (so that genotypes are the counts of reference alleles). If the reference
		//allele is '?', the whole SNP for that file were to set 'QQ'; 
		//sort the key according to the physical position, to get vsRsnum; 
		//Allocate memory for each individual, i.e., we have nCohort x nLoci; 
		 
			
		nLoci = 0; 
		map<string, int> mCohortRsQ; //cohort SNPs; 
		map<string, int> mrs2yes; 
		map<int, map<string, class mmSNP> > mfile2map; 
		vector<int> vindploid; 

		for (unsigned nf = 0; nf < vGin.size(); nf++)
		{   
			int filerows = 0; 
			int filecols = 0; 
			int filecols_diff = 0; 

			map<string, class mmSNP> rs2ss; //rs to snp summary; 
			
			ifstream infile; 
			streambuf * pbuf;
			char delimit[] = ";, \t";
			infile.open(vGin.at(nf).c_str(), ios::in);
			if(!infile.is_open()) 
			{
				fplog << "Quit: BIMBAM cannot open genotype file: " << vGin.at(nf) << endl; 
				cout << "-pimass: cannot open genotype file: " << vGin.at(nf) << endl; 
				safe_exit(); 
			} 
			pbuf = infile.rdbuf();
			// open file; 

			int ni = 0; 
			int ns = 0; 
			        
			string line; 
			line.assign(getline(pbuf)); 
			while(!line.empty())
			{   
				//printf("%s\n", line.c_str()); 
				filerows ++; 
				char *res = strtok((char*)line.c_str(), delimit); 
				if(res == NULL) break; 
				string rs(res);  
				if(end_pos > 0) 
				{
					map<string, long> :: iterator iter; 
					iter = mapRs2pos.find(rs); 
					if(iter == mapRs2pos.end() || iter->second < 0) 
					{
						line.assign(getline(pbuf));
						int len = min(100, (int)line.size()); 
						for (int i = 0; i < len; i++)
						{
							if((int)line.at(i) == 0) 
								line.at(i) = 'N'; 
						}                       
						continue; 
					}
				}   // rs; 

				mrs2yes[rs] = 1;
				
				pair<char, char> mm; 
				res = strtok(NULL, delimit); 
				if(res == NULL) 
				{
					cout << " error in input file ... " << endl; 
					exit(0); 
				}
				mm.first = res[0]; 
				res = strtok(NULL, delimit); 
				if(res == NULL) 
				{
					cout << " error in input file ... " << endl; 
					exit(0); 
				}
				mm.second = res[0];
					
				mapRs2mm[rs] = mm; 
				
				map<char, double> a2c;
				map<char, double> :: iterator iter; 
				vector<char> snpgt; 
				
				filecols = 0; 
				while(1) 
				{    
					double g0 = 0.0; 
					res = strtok(NULL, delimit); 
					if(res == NULL) break; 
					g0 = atof(res); 
					if(mode == 1) 
					{
						iter = a2c.find(mm.first); 
						if(iter == a2c.end())
							a2c[mm.first] = g0;
						else 
							iter->second += g0; 
						
						iter = a2c.find(mm.second); 
						if(iter == a2c.end())
							a2c[mm.second] = 2.0 - g0;
						else 
							iter->second += 2.0 - g0; 
						
					}    
					else 
					{
						cout << "-bibmam: wrong mode in read genotype distribution" << endl; 
						safe_exit(); 
					}
					
					filecols++; 
				}   // snp summary in a2c;    //A C G T ? + -

                if(ni == 0) ni = filecols;  
				if(filecols != ni) { 
					filecols_diff++; 
				    cout << rs << "\t" << filecols << "\t" << ni  << endl; 
					for (int i = 1; i < 20; i++)
						fplog << line.at(i) << " " << (int)line.at(i) << "  "; 
					fplog << endl; 
				}
				class mmSNP ss; 
				
				vector<char> type; 
				for (iter = a2c.begin(); iter != a2c.end(); ++iter)
				{
					if(iter->first != 'N') type.push_back(iter->first); 
				}
				
				if (type.size() == 0) 
				{
					fplog << "## Warning: In file " << vGin.at(nf) << " snp " << rs << " is null" << endl; 
				}
				else if (type.size() > 2) 
				{
					fplog << "## Warning: In file " << vGin.at(nf) << " snp " << rs << " has more than 2 alleles" << endl; 
					ss.A = 'X'; 
					ss.B = 'X'; 
				}
				else if(type.size() == 1) 
				{
					type.push_back('N'); 
					a2c['N'] = 0; 
				}
				
				else //if(a2c.size() == 2)  
				{
					double n0 = a2c[type.at(0)]; 
					double n1 = a2c[type.at(1)]; 
					
					if(n0 >= n1) 
					{
						ss.A = type.at(0); 
						ss.countA = n0; 
						ss.B = type.at(1); 
						ss.countB = n1; 
					}
					else 
					{
						ss.A = type.at(1); 
						ss.countA = n1; 
						ss.B = type.at(0); 
						ss.countB = n0; 
					}
				}   //translate a2c into mmSNP format; the first is the major; 
				a2c.clear(); 
				
				map<string, class mmSNP> :: iterator mmiter; 
				mmiter = rs2ss.find(rs); 
				if (mmiter != rs2ss.end())
				{
					fplog << "## WARNING: Two SNPs have the same rsnum " << mmiter->first << endl; 
					cout << "-pimass: two snp have same rsnum " << mmiter->first << endl; 
				}
				
				rs2ss.insert(pair<string, class mmSNP> (rs, ss)); 
		//		cout << rs << " " << rs2ss.size() << " " <<  ss.countA << " " << ss.countB << " " << ss.countQ << endl; 
				
				if(readpos == 0) 
				{
					mapRs2pos.insert( pair<string, long> (rs, (long) rs2ss.size()));  
					mapRs2chr.insert( pair<string, int> (rs, 0)); 
				}
				//if no rs-pos file and only one genotype file, use snp order as pos.
				line.assign(getline(pbuf));
			}

			infile.close();
			mfile2map[nf] = rs2ss; 
			
			ns = filerows; 
            ni = filecols;  
			
			if(filecols_diff > 0)
			{
				fplog << "## WARNING: " << filecols_diff << " SNPs has different # of ind's from the first line" << endl; 
				cout << "-pimass: in " << filecols_diff << " snps, individual differ from the first line" << endl; 
			}
			
			{
				vFileIndiv.push_back(ni);
				nCohort += ni; 
			}
			fplog << "## pimass: File " << nf << " has " << vFileIndiv.at(nf) << " ind's and " \
				<< rs2ss.size() << " SNPs" << endl; 
			cout << "-pimass: file " << nf << " has " << vFileIndiv.at(nf) << " individual and " \
				<< rs2ss.size() << " snps" << endl; 
			rs2ss.clear(); 
		}   // SNP summary data;
		
//		{
//			map<string, class mmSNP> :: iterator iter; 
//			for (int nf = 0; nf < mfile2map.size(); nf++)
//			{
//				for (iter = mfile2map[nf].begin(); iter != mfile2map[nf].end(); ++iter)
//				{
//					cout << nf << "\t" <<  iter->first << "\t" << iter->second.A << iter->second.B; 
		//          cout << "\t" << iter->second.countA << "\t" << iter->second.countB;   
//					cout << "\t" << iter->second.countQ << endl; 
//				}
//			}
//		}   //test if the mmSNP build ok; 
		////////////////////////////////////////////////////////////////////////////
		// now perform merger;   //snp by snp; 

		map<string, int> ::iterator m_iter; 
		map<string, vector<char> > mrs2coding;   //rs to reference allele coding for each file;  
		for (m_iter = mrs2yes.begin(); m_iter != mrs2yes.end(); ++m_iter)
		{
			string rs; 
			rs.assign(m_iter->first); 
			vector<char> coding; 
			class mmSNP ref; 
			int bingle = 0; 
			for (unsigned nf = 0; nf < vGin.size(); nf++)
			{
				char cr; 
				map<string, class mmSNP> :: iterator iter; 
				iter = mfile2map[nf].find(rs); 
				if(iter == mfile2map[nf].end())  
					cr = 'X'; 
				else if(ref.countA + ref.countB < 1)  // first one; 
				{
					ref.assign(iter->second); 
					cr = ref.A;
				}
				else 
					cr = merge_mmSNP(ref, iter->second);
				if(cr == 'X') bingle = 0; 

				coding.push_back(cr); 
			}
			//here coding allele is major allele, because it's convenient for monomorphic snp. 

			if (bingle == 1) 
				m_iter->second = 0; //remove this SNP; 
			
			mrs2coding[rs] = coding;                  
			mapRs2mm[rs] = pair<char, char> (ref.A, ref.B); 
			double countAB = ref.countA + ref.countB; 
			double missrate = ref.countQ / (countAB + ref.countQ); 
			//cout << missrate << "\t" << ref.countA << "\t" << ref.countB << "\t" << ref.countQ << endl; 
			if( missrate > 0)        //missing test; 
				mrs2yes[rs] = -1; 
			else if(countAB == 0) 
			{
				mapRs2maf[rs] = 0.0;
				mapRs2var[rs] = 1e-10; 
			}
			else 
			{
				double af = (double) (ref.countB) / countAB;
				mapRs2maf[rs] = af;   
				mapRs2var[rs] = 2.0 * af * (1.0 - af); 
//				if(af < 0) cout << ref.countB << " " << countAB << " " << af << endl; 
				//grant; 
			}
			//here A is major, B is minor. 
			vector<char> ().swap(coding); 
		}
		
		int count_pos = 0;  //failed to have a position; 
		int count_am = 0;   //failed to match allele type; 
		int count_maf = 0;  //failed due to maf too extreme; 
		int count_miss = 0; 
		vector<pair<string, pair<long, int> > > vp;
		map<string, long> :: iterator pos_iter; 
		map<string, int> :: iterator chr_iter; 
		for (m_iter = mrs2yes.begin(); m_iter != mrs2yes.end(); m_iter++)
		{
			string rs(m_iter->first);
			if(m_iter->second == -1) 
			{
				count_miss ++; 
				continue; 
			}
			if(m_iter->second == 0) 
			{
				count_am ++; 
				continue; 
			}

//			fplog << mapRs2maf[rs] << endl; 
			if(m_not_snp == 0 && mapRs2maf[rs] <= m_exclude_maf) 
			{
//				cout << mapRs2maf[rs] << endl; 
				//grant; 
				count_maf ++;    
				m_iter->second = 0;  //mark this SNP to be excluded. 
				continue; 
			}
			if(m_exclude_nopos && mapRs2pos[rs] == 0)   //if snp no position; 
			{
				count_pos ++; 
				m_iter ->second = 0; //mark this SNP to be excluded. 
				continue; 
			}
			pair<string, pair<long, int> > tmp;
			tmp.first.assign(m_iter->first);
			pos_iter = mapRs2pos.find(m_iter->first);
			if(pos_iter != mapRs2pos.end())
				tmp.second.first = pos_iter->second;
			else 
				tmp.second.first = 0; 

			chr_iter = mapRs2chr.find(tmp.first); 
			if(chr_iter != mapRs2chr.end())
				tmp.second.second = chr_iter->second; 
			else
				tmp.second.second = 0; 
									   
			vp.push_back(tmp); 
		}
		if(count_miss > 0) 
		{
			fplog << "## pimass: Exclude " << count_miss << " SNPs due to miss genotypes " << endl;
			cout << "-pimass: exclude " << count_miss << " snps due to missing genotypes "  << endl;
		}
		if(count_am > 0) 
		{
			fplog << "## pimass: Exclude " << count_am << " SNPs due to failure to match betweeen files." << endl;
			cout << "-pimass: exclude " << count_am << " snps due to failure to match betweeen files." << endl;
		}
		if(count_maf > 0) 
		{
			fplog << "## pimass: Exclude " << count_maf << " SNPs due to maf <= " << m_exclude_maf << endl;
			cout << "-pimass: exclude " << count_maf << " snps due to maf <= " << m_exclude_maf << endl;
		}
		if(count_pos > 0) 
		{
			fplog << "## pimass: Exclude " << count_pos << " SNPs due to no position information" << endl;
			cout << "-pimass: exclude " << count_pos << " snps due to no position information" << endl;
		}
		stable_sort(vp.begin(), vp.end(), rscomp); 
		//sort the rs pos vector in order of chr && pos;
		map<string, int> mrs2index; //the column of each snp; 
		vsRsnum.clear(); 
		for(int i = 0; i < (int) vp.size(); i++)
		{
			vsRsnum.push_back(vp.at(i).first);
			mrs2index[vp.at(i).first] = i; 
		}
		vector<pair<string, pair<long, int> > >().swap(vp); 
		mrs2yes.clear(); 
		nLoci = (long) vsRsnum.size(); 
		if(nLoci == 0) 
		{
			cout << "-pimass: no valid snp" <<  endl; 
			safe_exit(); 
		}
        //now vsRsnum hold ID of all valid SNPs in order; 
		
		mCohortRsQ.clear(); 
////////////////////////////////////////////////////////////////////////////////////
		
	   	snpmgt = Allocate2DMatrix((long int) nCohort, nLoci); 
//		snpmgt = gsl_matrix_alloc(nCohort, nLoci); 
		
////////////////////////////////////////////////////////////////////////////////////
		// go over genotype files again; 
		// for each diploid file, scan each SNP, fill in the genotype table. 
		int start_ni = 0; 
		for (int nf = 0; nf < (int)vGin.size(); nf++)
		{       
			streambuf * pbuf;
			ifstream infile; 
			char delimit[] = ";, \t";
			infile.open(vGin.at(nf).c_str(), ios::in);
			if(!infile.is_open()) 
			{
				cout << "-pimass: cannot open genotype file: " << vGin.at(nf) << endl; 
				safe_exit(); 
			} 
			pbuf = infile.rdbuf();
			 
			string line; 
			line.assign(getline(pbuf)); 
			//possible individual ids
				
			while(line.size() > 0)
			{   
				char * res = strtok((char*)line.c_str(), delimit); 
				if(res == NULL) break; 
				string rs; 
				rs.assign(res);  
				if(end_pos > 0) 
				{
					map<string, long> :: iterator iter; 
					iter = mapRs2pos.find(rs); 
					if(iter == mapRs2pos.end() || iter->second < 0) 
					{
						line.assign(getline(pbuf));
						fplog << line.c_str() << endl; 
						//grant; 
						continue; 
					}
				}    
				int m = -1; 
				map<string, int> :: iterator s2i; 
				s2i = mrs2index.find(rs); 
				if(s2i == mrs2index.end())
				{
					line.assign(getline(pbuf));
					continue; 
				}
				else 
				 	m = s2i->second;  
				// rs number; 
				res = strtok(NULL, delimit); 
				char aA = res[0]; 
				res = strtok(NULL, delimit); 
				//skip 2nd char; 

				char ref = mrs2coding[rs].at(nf); 
				int ni = start_ni; 
				for (int i = 0; i < vFileIndiv.at(nf); i++)
				{    
					double g0 = -1.0; 
					char * res = strtok(NULL, delimit); 
					if(res == NULL) break; 
					g0 = atof(res); 
				
					
					if(1) 
					{
						if(ref == aA) 
							snpmgt[ni][m] = 2.0 - g0;  
						else 
							snpmgt[ni][m] = g0;  
					}
					ni++; 
				}
				line.assign(getline(pbuf));
			}
			infile.close();
			cout << "-pimass: read file " << nf << " again " << endl; 
			start_ni += vFileIndiv.at(nf); 
		}

		
		fplog << "## number genotype files = " << vGin.size() << endl; 
		fplog << "## number phenotype files = " << vPin.size() << endl; 
		fplog << "## number of cohort = " << nCohort << endl; 
		fplog << "## number of snp = " << nLoci  << endl;
	}

//////////////////////////////////////////////////////////////////////////////////////////
    
//	for (int m = 0; m < nLoci; m++)
//	{
//		for (int i = 0; i < nCohort; i++)
//			fplog << snpmgt[i][m] << " "; 
//	    fplog << endl; 
//	}
	
//	cout << m_exclude_maf << endl; 
	return 1; 
}  

int ModelnData::read_bimbam_phenotype(int mcmc)
{
	if(vPin.size() == 0 || vPin.size() != vGin.size()) 
		return 0; 
	vector<vector<real> > trans_phval; 
#if defined (MPI_ENABLED)
	real * tmp_phval; 
	if(procID == 0)
#endif
	{
		fstream infile; 
		streambuf * pbuf; 
		string sfn; 
		char delimit[] = ",; :\t";
			   
		int cur_ni = 0; 
		for (int nf = 0; nf < (int) vPin.size(); nf++)
		{   
			int ni = vFileIndiv.at(nf);
			sfn.assign(vPin.at(nf)); 
			
			if(sfn.compare("1") == 0)   //case; 
			{
				for (int i = 0; i < ni; i++)
				{
					vector<real> ind_phval; 
					for (int np = 0; np < nPH; np++)
						ind_phval.push_back(1); 
					trans_phval.push_back(ind_phval);
				}
				cur_ni += ni; 
				continue; 
			}
			
			else if(sfn.compare("z") == 0)  //control; 
			{
				for (int i = 0; i < ni; i++)
				{
					vector<real> ind_phval; 
					for (int np = 0; np < nPH; np++)
						ind_phval.push_back(0); 
					trans_phval.push_back(ind_phval);
				}
				cur_ni += ni; 
				continue; 
			}
			
			infile.open(sfn.c_str(), ios::in);
			if(!infile.is_open())
			{
				fplog << "ERROR: BIMBAM cannot open phenotype file " << nf << endl; 
				cout << "-pimass: cannot open phenotype file " << nf << endl; 
				safe_exit(); 
			}
			pbuf = infile.rdbuf();
			string line;
			
//			for (int i = 0; i < vFileIndiv.size(); i++)
//				cout << vFileIndiv.at(i) << endl; 
//			cout << endl; 
			 
			for (int i = 0; i < ni; i++)
			{
				vector<real> ind_phval; 
				line.assign(getline(pbuf)); 
				char * res = strtok((char*)line.c_str(), delimit); 
				for (int np = 0; np < nPH; np++)
				{
					if(res == NULL) break; 
					string sv(res);
					if(sv.compare("NA") == 0) ind_phval.push_back(NA);
					else ind_phval.push_back(atof(sv.data()));
					res = strtok(NULL, delimit); 
				}

				if ((int) ind_phval.size() != nPH)
				{
					nPH = ind_phval.size(); 
					fplog << "WARNING: File " << nf << " phenotypes columns smaller than specified." << endl;
					cout << "-pimass: file " << nf << " phenotypes columns smaller than specified." << endl;
				}
					
				trans_phval.push_back(ind_phval);
				ind_phval.clear();
			}
			line.assign(getline(pbuf));
			if(line.size() > 0) 
			{
				fplog << "WARNING: Phenotypes in file " << nf << " larger than number of indiv." << endl;
				cout << "-pimass: phenotypes in file " << nf << " larger than number of individuals" << endl;
			}             
			if ((int)trans_phval.size() - cur_ni < ni) 
			{
				fplog << "QUIT: Phenotypes in file " << nf << " smaller than number of indiv." << endl;
				cout << "-pimass: phenotypes in file " << nf << " smaller than number of individuals" << endl;
                safe_exit(); 
			}
			infile.close(); 
			cur_ni += ni; 
		}
	}
	
//	cout << "trans_phval" <<  trans_phval.size() << endl; 
#if defined (MPI_ENABLED)
   	tmp_phval = new double[nPH * nCohort];
	if (procID == 0) 
	{
		for (int np = 0; np < nPH; np++)
			for (int i = 0; i < nCohort; i++)
				tmp_phval[i * nPH + np] = trans_phval.at(i).at(np); 
	}
	
	MPI_Barrier(MPI_COMM_WORLD); 
	MPI_Bcast(tmp_phval, nPH * nCohort, MPI_DOUBLE, 0, MPI_COMM_WORLD); 

	for(int np = 0; np < nPH; np++)
	{
		vector<int> vindex; 
		vector<real> vt;
		int ni = 0; 
		for(int i = 0; i < nCohort; i++)
		{
			if (fabs(tmp_phval[i*nPH+np] - NA) < 0.1)
			{
				ni++; 
			} 
			else 
			{
				vindex.push_back(ni++); 
				vt.push_back(tmp_phval[i * nPH + np]);
			}
		}
		vv_phval.push_back(vt);
		vv_phdex.push_back(vindex);
		vindex.clear(); 
		vt.clear();
	}
	
#else 
	
	for(int np = 0; np < nPH; np++)
	{
		vector<int> vindex; 
		vector<real> vt; 
		int ni = 0; 
		for(int i = 0; i < nCohort; i++)
		{
			if (fabs(trans_phval.at(i).at(np) - NA) < 0.1)
			{
				ni++; 
				vt.push_back(trans_phval.at(i).at(np));
			} 
			else 
			{
				vindex.push_back(ni++); 
				vt.push_back(trans_phval.at(i).at(np));
			}
		}
		vv_phval.push_back(vt);
		vv_phdex.push_back(vindex);
		vindex.clear(); 
		vt.clear();
	}   //vv_phdex contains "index" of indiv that has valid phval. 
#endif	

	
#if defined (MPI_ENABLED)
	delete[] tmp_phval; 
	if (procID == 0)
#endif  
	{
		vector<vector<real> >().swap(trans_phval); 
//		cout << "-pimass: number of phenotypes = " << nPH << endl; 
	}
	fplog << "### number of phenotypes = " << vv_phval.at(0).size() << endl; 
	
//	fplog << "### index of individual with vaild phenotypes "; 
//	for (unsigned i = 0; i < vv_phdex.at(0).size(); i++)
//		fplog << vv_phdex.at(0).at(i) << " "; 
//	fplog << endl; 

//	for (int i = 0; i < vv_phdex.at(0).size(); i ++)
//	{
//		int j = vv_phdex.at(0).at(i); 
//		fplog << vv_phval.at(0).at(j) << " "; 
//	}
//	fplog << endl; 
	return 1; 
}

bool bfcomp(pair<real, int> a, pair<real, int> b)
{
	return (a.first > b.first); 
}

void ModelnData::open_log(void)
{
#if defined (MPI_ENABLED)
	if(procID == 0)
#endif
	{
		string sfn("output/");
		sfn.append(fnOutput);
		sfn.append(".log.txt");
		fplog.open(sfn.c_str(), ios::out);
		if(!fplog.is_open()) 
		{
			cout << "-pimass: cannot open log file" << endl;
//			safe_exit(); 
		}
	}
}

void ModelnData::close_log(void)
{
#if defined (MPI_ENABLED)
	if(procID == 0)
#endif
		fplog.close();
}

real max(real a, real b)
{
	return (a > b ? a:b); 
}



cMCMC::cMCMC()
{
	;                                  
}

cMCMC::~cMCMC()
{
	;
}

cMCMC::cMCMC(real hh, real pp, real qq, real ll, vector<int>& ss)
{
	h = hh; p = pp; q = qq; l = ll; 
	sin = ss; 
}                                       

void cMCMC::assign(class cMCMC& s) 
{
	h = s.h; p = s.p; q = s.q; l = s.l; 
	sin = s.sin; 
	beta = s.beta; 
	sigma = s.sigma; 
//	gin.clear();
//	gin.insert(gin.end(), s.gin.begin(), s.gin.end()); 
//	sin.clear(); 
//	sin.insert(sin.end(), s.sin.begin(), s.sin.end()); 
	hhat = s.hhat; bf = s.bf; post = s.post;
	like = s.like; 
}

//void ModelnData::approx_inverse(gsl_matrix * m, int d, double& det)
//{
//	double mmin = 1e100; 
//	for (int i = 0; i < d; i++)
//	{
//		double temp = gsl_matrix_get(m, i, i); 
//		if(temp < mmin) 
//			mmin = temp; 
//	}   //smallest entry in diag. 
//	
//	double eps = 0.1 * mmin; 
//	vector<int> taken; 
//	taken.insert(taken.begin(), d, 0); 
//	int grp = 1; 
//	
//	for (int i = 0; i < d; i++)
//	{   
//		if(taken.at(i) == 0) 
//			taken.at(i) = grp;  
//		else 
//			continue; 
//		for (int j = i+1; j < d; j++)
//		{
//			if(taken.at(j) != 0) continue; 
//			double temp = gsl_matrix_get(m, i, j); 
//			if (temp > eps) 
//				taken.at(j) = grp; 
//		}
//	}
//		
//}


//documented by C P Robert stat-comp 95 5, 121-125; 
double ModelnData::get_truncated_gaussian(double mean, int lg0) 
{
	double res = 0; 
//	double ratio = 1; 
//	double alpha = 0; 
	if(lg0 == 1) //truncate from left; 
	{
//		//simulate N(0,1) left truncate at -mean; 
//		alpha = (sqrt(mean * mean + 4.0) - mean) / 2.0; 
//		do  
//		{
//			double unif = gsl_rng_uniform(gsl_r); 
//			res = -log(unif) * alpha - mean; 
//			double temp = res - alpha; 
//			ratio = exp(-temp * temp / 2.0); 
//		} while(gsl_rng_uniform(gsl_r) > ratio); 
//		res += mean; 
		res = gsl_ran_ugaussian_tail(gsl_r, -mean) + mean; 
	}
	else if(lg0 == 0) //truncate from right; 
	{
		//simulate z~N(0,1) right truncated at mean; 
		//which is equivalent to simulate -z that is left truncated at mean; 
//		alpha = (sqrt(mean * mean + 4.0) + mean) / 2.0; 
//		do  
//		{
//			double unif = gsl_rng_uniform(gsl_r); 
//			res = -log(unif) * alpha + mean; 
//			double temp = res - alpha; 
//			ratio = exp(-temp * temp / 2.0); 
//		} while(gsl_rng_uniform(gsl_r) > ratio); 
//		res = mean-res; 
		res =  mean - gsl_ran_ugaussian_tail(gsl_r, mean); 
	}	

	return (res); 
}

void ModelnData::get_gXX(class cMCMC& cur, class cMCMC& s, gsl_matrix * gXX) 
{
	int ns = (int) s.sin.size(); 
	int ni = nCohort; 

//	vector<int> newindex; 
//	vector<int> oldindex; 
//	for(int i = 0; i < ns; i++)
//		oldindex.push_back(i); 
//	map<long, int> map_cur2index; 
//	for (int i = 0; i < cur.sin.size(); i++)
//		map_cur2index[cur.sin.at(i)] = i; 
//
//	for (int i = 0; i < ns; i++)
//	{
//		map<long, int> :: iterator iter; 
//		iter = map_cur2index.find(s.sin.at(i)); 
//		if(iter == map_cur2index.end())
//			newindex.push_back(i); 
//		else 
//			oldindex.push_back(i); 
//	}
//
//    if(newindex.size() > 0) 
//	{   
//		struct timeval beg, end;
//		struct timezone tz;
//		int getb = gettimeofday(&beg, &tz);
//
//		gsl_matrix * mA = NULL, *mB = NULL, *mC = NULL; 
//		mA = gsl_matrix_alloc(ni, ns); 
//		for (int j = 0; j < ns; j++)
//		{
//			gsl_vector_view gv = gsl_matrix_column(snpmgt, s.sin.at(j)); 
//			gsl_matrix_set_col(mA, j, &gv.vector); 
//		}
//
//		int news = newindex.size(); 
//		mB = gsl_matrix_alloc(ni, news);
//		for (int j = 0; j < news; j++)
//		{
//			gsl_vector_view gv = gsl_matrix_column(snpmgt, s.sin.at(newindex.at(j))); 
//			gsl_matrix_set_col(mB, j, &gv.vector); 
//		}
//
//		char cn ='N'; 
//		char ct = 'T'; 
//		mC = gsl_matrix_alloc(ns, news); 
//		lapack_dgemm(&ct, &cn, 1.0, mA, mB, 0.0, mC); 
//
//		for (int i = 0; i < ns; i++)
//		{
//			for (int j = 0; j < news; j++)
//			{
//				double temp =  gsl_matrix_get(mC, i, j);
//				
//				int tj = newindex.at(j); 
//				gsl_matrix_set(gXX, i, tj, temp);  
//				gsl_matrix_set(gXX, tj, i, temp);  
//				long k1 = s.sin.at(i); 
//				long k2 = s.sin.at(tj); 
//				pair<long, long> key; 
//				if(k1<k2) {key.first=k1; key.second=k2;}
//				else {key.first=k2; key.second=k1;}
//				m_recentxtx[key] = temp; 
//			}
//		}
//
//
//		gsl_matrix_free(mA); 
//		gsl_matrix_free(mB); 
//		gsl_matrix_free(mC); 
//
//		int gete = gettimeofday(&end, &tz);
//		if(getb == 0 && gete == 0) 
//			m_clock3 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
//	}
//
//	map<pair<long, long>, double> :: iterator iter; 
//	for (int ti = 0; ti < oldindex.size(); ti++)
//	{                       
//		int i = oldindex.at(ti); 
//		long m1 = s.sin.at(i); 
//		for (int tj = ti; tj < oldindex.size(); tj++)
//		{
//			double sum = 0; 
//			int j = oldindex.at(tj); 
//			long m2 = s.sin.at(j); 
//			pair<long, long> key;  
//			if(m1 < m2) {key.first=m1; key.second=m2;} 
//			else {key.first=m2; key.second=m1;}  
//			iter = m_recentxtx.find(key); 
//			m_count1+=1; 
//			if(iter == m_recentxtx.end())
//			{   
//				m_count2+=1; 
//					
////				struct timeval beg, end;
////				struct timezone tz;
////				int getb = gettimeofday(&beg, &tz);
//
//				gsl_vector_view va = gsl_matrix_column(snpmgt, m1); 
//				gsl_vector_view vb = gsl_matrix_column(snpmgt, m2); 
//				lapack_ddot(&va.vector, &vb.vector,&sum); 
//
////				int gete = gettimeofday(&end, &tz);
////				if(getb == 0 && gete == 0) 
////					m_clock3 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
//
//				m_recentxtx[key] = sum; 
//			}
//			else 
//   				sum = iter->second; 
//
//			gsl_matrix_set(gXX, i, j, sum);  
//			if(i != j) 
//				gsl_matrix_set(gXX, j, i, sum);  
//		}
//	}


//	gsl_matrix * mA = gsl_matrix_alloc(ni, ns); 
//	for (int j = 0; j < ns; j++)
//	{
//		int m = s.sin.at(j); 
//		for (int i = 0; i < ni; i++)
//			gsl_matrix_set(mA, i, j, (double) snpmgt[i][m]); 
////		gsl_vector_view gv = gsl_matrix_column(snpmgt, s.sin.at(j)); 
////		gsl_matrix_set_col(mA, j, &gv.vector); 
//	}
	
	real ** mA = Allocate2DMatrix(ni, ns); 
	for (int j = 0; j < ns; j++)
	{
		int m = s.sin.at(j); 
		for (int i = 0; i < ni; i++)
			mA[i][j] = snpmgt[i][m]; 
	}

	struct timeval beg, end;
	struct timezone tz;
	int getb = gettimeofday(&beg, &tz);

	map<pair<long, long>, double> :: iterator iter; 
	for (int i = 0; i < ns; i++)
	{                       
		long m1 = s.sin.at(i); 
		for (int j = i; j < ns; j++)
		{
			double sum = 0; 
			long m2 = s.sin.at(j); 
			pair<long, long> key;  
			if(m1 < m2) {key.first=m1; key.second=m2;} 
			else {key.first=m2; key.second=m1;}  
			iter = m_recentxtx.find(key); 
			m_count1+=1; 
			if(iter == m_recentxtx.end())
			{   
				m_count2+=1; 
					
				struct timeval beg, end;
				struct timezone tz;
				int getb = gettimeofday(&beg, &tz);

//				gsl_vector_view va = gsl_matrix_column(mA, i); 
//				gsl_vector_view vb = gsl_matrix_column(mA, j); 
//				lapack_ddot(&va.vector, &vb.vector,&sum); 

				for (int k = 0; k < ni; k++)
					sum += (double) (mA[k][i] * mA[k][j]); 

				int gete = gettimeofday(&end, &tz);
				if(getb == 0 && gete == 0) 
					m_clock3 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 

				m_recentxtx[key] = sum; 
			}
			else 
   				sum = iter->second; 

			gsl_matrix_set(gXX, i, j, sum);  
			if(i != j) 
				gsl_matrix_set(gXX, j, i, sum);  
		}
	}
	int gete = gettimeofday(&end, &tz);
	if(getb == 0 && gete == 0) 
	{
		m_clock2 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
		m_clock4 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
	}

	if(m_recentxtx.size() > m_bufsize) 
	{
		map<pair<long, long>, double> ().swap(m_recentxtx); 
		for (int i = 0; i < ns; i++)
		{
			long m1 = s.sin.at(i); 
			for (int j = i; j < ns; j++)
			{
				long m2 = s.sin.at(j); 
				pair<long, long> key; 
				if(m1 < m2) {key.first=m1; key.second=m2;} 
				else {key.first=m2; key.second=m1;}  
				m_recentxtx[key] = gsl_matrix_get(gXX, i, j); 
			}
		}
	}

//	gsl_matrix_free(mA); 
	Free2DMatrix(mA); 
}

void ModelnData::calc_hh(class cMCMC& s) 
{
	double ex1 = 0;
	double ex2 = 0; 
	for (int i = 0; i < nCohort; i++)
	{
		double mp = 0; 
		for (unsigned j = 0; j < s.sin.size(); j++)
		{
			int loc = s.sin.at(j); 
			mp +=  s.beta.at(j) * snpmgt[i][loc]; 
		}
		ex1 += mp;
		ex2 += mp * mp; 
	}
	ex1 /= nCohort; 
	double var = ex2 / nCohort - ex1 * ex1; 
	s.hhat = var / (var + 1.0); 
}

double sumlog(double a, double b)
{
	double c = a > b ? a : b;
	return(c + log(exp(a-c) + exp(b-c))); 
}

//void ModelnData::calc_post(class cMCMC& s, gsl_vector * curZ)
//{
//	if(s.sin.size() == 0) 
//	{
//		s.sigma = 0; 
//		s.bf = 0;     
//		s.hhat = 0; 
//		double ss = (double) s.sin.size(); 
//		s.like = (ss - 1.0) * log(s.p) + (nLoci - ss) * log(1.0 - s.p); 
//		s.beta.clear(); 
//		for (unsigned i = 0; i < s.sin.size(); i++)
//			s.beta.push_back(0); 
//		return;        
//	}
//	
//	double suma = 0.0;
//	map<string, real> :: iterator iter; 
//	
//	if(m_not_snp == 1) 
//		suma = (double) s.sin.size(); 
//	else 
//	{
//		for (unsigned i = 0; i < s.sin.size(); i++)
//		{
//			string rs;
//			rs.assign(vsRsnum.at(s.sin.at(i))); 
//			iter = mapRs2var.find(rs); 
//			if(iter == mapRs2var.end())
//			{
//				cout << "-mcmc: snp has no maf: " << rs << endl; 
//				safe_exit(); 
//			}
//			real var = iter->second; 
//			suma +=  var; 
//		}
//	}
//	if(suma < 1e-100) suma = 1e-100;  
//	double bfp = sqrt(s.h / (1.0 - s.h) / suma); 
//	if(bfp < 1e-3) bfp = 1e-3; 
//	else if(bfp > 1e6) bfp = 1e6; 
//    s.sigma = bfp; 
//	
////	s.sigma=FIXED_SIGMA; //grant:fixed_parameter;  
//
///////////////////////////////////////////////////////////////////////////////////////////	
//	
//	////////////////////////////////////////////////////////////////////////////////////
//	int ni = nCohort; 
//	int ns = s.sin.size(); 
//	int col = ns; 
//	int row = ni + ns; 
//	
//	gsl_vector * gph = gsl_vector_alloc(row);
//	for (int i = 0; i < ni; i++)
//		gsl_vector_set(gph, i, gsl_vector_get(curZ, i)); 
//	for (int i = ni; i < row; i++)
//		gsl_vector_set(gph, i, 0); 
//	if(cc == 1) 
//		center(ni, gph); //cc == 0; only need to center once, and already being done in mcmc().  
//	
//	gsl_vector * gyx = gsl_vector_alloc(col);
//	for (int j = 0; j < col; j++)
//	{
//		int m = s.sin.at(j); 
//		double temp = 0; 
//		if(cc == 1) 
//		{
//			for (int i = 0; i < ni; i++)
//				temp += snpmgt[i][m] * gsl_vector_get(gph, i); 
//		}
//		else 
//			temp = m_xty[m]; 
//		gsl_vector_set(gyx, j, temp); 
//	}   //XtY; 
//	
//	double sg = 1.0 / s.sigma; //sudo genotype; 
//
//	struct timeval beg, end;
//    struct timezone tz;
//    int getb, gete;
//    getb = gettimeofday(&beg, &tz);
//
//	gsl_matrix * eA = gsl_matrix_alloc(row, col); //extended design matrix; 
//	for (int j = 0; j < col; j++)
//	{
//        int m = s.sin.at(j); 
//		for (int i = 0; i < ni; i++)
//            gsl_matrix_set(eA, i, j, snpmgt[i][m]); 
//		for (int i = ni; i< row; i++)
//		{
//			if (i == (ni +j)) 
//            	gsl_matrix_set(eA, i, j, sg); 
//			else 
//            	gsl_matrix_set(eA, i, j, 0); 
//		}
//	}
//	// setup the extended design matrix; 
//
//	gsl_matrix * mV = gsl_matrix_alloc(col, col); 
//	gsl_vector * vS = gsl_vector_alloc(col); 
//	gsl_vector * work = gsl_vector_alloc(col); 
//	gsl_vector * vbeta = gsl_vector_alloc(col); 
//
//	gsl_linalg_SV_decomp(eA, mV, vS, work);  
//    gsl_linalg_SV_solve(eA, mV, vS, gph, vbeta); 
//
//
//	double logdet = 0; 
//	for (int i = 0; i < col; i++)
//		logdet += log(fabs(gsl_vector_get(vS, i))); 
//	logdet *= 2.0; 
//	
//   	double bob = 0;
//	gsl_blas_ddot(vbeta, gyx, &bob); 
//	
//	double yy = 0; 
//	gsl_blas_ddot(gph, gph, &yy); 
//	
//	double tlast = (yy - bob) / (yy);     //note this assumes that gph is centered.
//	double logbf = -0.5 * logdet - ns * log((double) s.sigma) - ni * 0.5 * log((double)tlast); // + 0.5 * log((double)ni); 
//	s.bf = logbf; 
//
//	gete = gettimeofday(&end, &tz);
//	if(getb == 0 && gete == 0)
//	    m_clock1 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec);
///////////////////////////////////////////////////////////////////
//	
//	s.beta.clear(); 
//	for (int i = 0; i < col; i++)
//		s.beta.push_back(gsl_vector_get(vbeta, i)); 
//	//this beta is normalized by tau; so that tau = 1; 
//    
//	gsl_matrix_free(eA);
//	gsl_matrix_free(mV);
//	gsl_vector_free(vS);
//	gsl_vector_free(vbeta);
//	gsl_vector_free(gph);
//	gsl_vector_free(gyx);
//
//
//	double loglike = 0;
//	double ss = (double) (s.sin.size()); 
//	loglike += (ss - 1.0) * log(s.p) + (nLoci - ss) * log(1.0 - s.p); 
//	// the above calc prior for the model. 
//	s.like = loglike; 
//}

void ModelnData::calc_post(class cMCMC& cur, class cMCMC& s, gsl_vector * curZ)
{
	if(s.sin.size() == 0) //testing; 
	{
		s.sigma = 0; 
		s.bf = 0;     
		s.hhat = 0; 
		double ss = (double) s.sin.size(); 
		s.like = (ss - 1.0) * log(s.p) + (nLoci - ss) * log(1.0 - s.p); 
		s.beta.clear(); 
		for (unsigned i = 0; i < s.sin.size(); i++)
			s.beta.push_back(0); 
		return;        
	}
	
	double suma = 0.0;
	map<string, real> :: iterator iter; 
	
	if(m_not_snp == 1) 
		suma = (double) s.sin.size(); 
	else 
	{
		for (unsigned i = 0; i < s.sin.size(); i++)
		{
			string rs;
			rs.assign(vsRsnum.at(s.sin.at(i))); 
			iter = mapRs2var.find(rs); 
			if(iter == mapRs2var.end())
			{
				cout << "-mcmc: snp has no maf: " << rs << endl; 
				safe_exit(); 
			}
			real var = iter->second; 
			suma +=  var; 
		}
	}
	if(suma < 1e-100) suma = 1e-100;  
	double bfp = sqrt(s.h / (1.0 - s.h) / suma); 
	if(bfp < 1e-3) bfp = 1e-3; 
	else if(bfp > 1e6) bfp = 1e6; 
    s.sigma = bfp; 
	
//	s.sigma=FIXED_SIGMA; //grant:fixed_parameter;  

/////////////////////////////////////////////////////////////////////////////////////////	
	
	////////////////////////////////////////////////////////////////////////////////////
	int ni = nCohort; 
	int ns = s.sin.size(); 
	int col = ns; 
	
	gsl_vector * gph = gsl_vector_alloc(ni); 
	gsl_vector_memcpy(gph, curZ); 
	if(cc == 1) 
		center(ni, gph); //cc == 0; only need to center once, and already being done in mcmc().  
	
	gsl_vector * gyx = gsl_vector_alloc(col);
	for (int j = 0; j < col; j++)
	{
		int m = s.sin.at(j); 
		double temp = 0; 
		if(cc == 1) 
		{
			for (int i = 0; i < ni; i++)
				temp += snpmgt[i][m] * gsl_vector_get(gph, i); 
		}
		else 
			temp = m_xty[m]; 
		gsl_vector_set(gyx, j, temp); 
	}   //XtY; 
	
	double inv_va = 1./(s.sigma * s.sigma); 

	gsl_matrix * ginvOmega = gsl_matrix_alloc(col, col); 
	gsl_vector * res = gsl_vector_alloc(col); 
	
	struct timeval beg, end;
    struct timezone tz;
    int getb, gete;
    getb = gettimeofday(&beg, &tz);
	
	get_gXX(cur, s, ginvOmega);
	gete = gettimeofday(&end, &tz);
	if(getb == 0 && gete == 0)
	    m_clock1 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec);

	
	for (int i = 0; i < col; i++)
		gsl_matrix_set(ginvOmega, i, i, gsl_matrix_get(ginvOmega, i,i) + inv_va); 

    getb = gettimeofday(&beg, &tz);
	gsl_set_error_handler_off(); 
	int status = gsl_linalg_cholesky_decomp(ginvOmega);
	if(status == GSL_EDOM) 
	{
		fplog << "### non-positive definite matrix, ns=" << ns << " sigma=" << " inv_va=" << inv_va << endl; 
		cout << "### non-positive definite matrix, ns=" << ns << " sigma=" << " inv_va=" << inv_va << endl; 
		for (int i = 0; i < ns; i++)
			cout << s.sin.at(i) << " "; 
		cout << endl;
		exit(0); 
	}
	gsl_linalg_cholesky_solve(ginvOmega, gyx, res); //res = inv(ginvOmega) %*% gyx; 
                      
//	char trans[] = "N"; 
//	lapack_LU_vsolve (trans, ginvOmega, res, gyx); 

	gete = gettimeofday(&end, &tz);
//	if(getb == 0 && gete == 0)
//	    m_clock2 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec);

	double logdet = 0; 
	for (int i = 0; i < col; i++)
		logdet += log(fabs(gsl_matrix_get(ginvOmega, i, i))); 
	logdet *= 2.0; 
	
   	double bob = 0;
	gsl_blas_ddot(res, gyx, &bob); 
	
	double yy = 0; 
	gsl_blas_ddot(gph, gph, &yy); 
	
	double tlast = (yy - bob) / (yy);     //note this assumes that gph is centered.
	double logbf = -0.5 * logdet + ns * 0.5 * log((double) inv_va) - ni * 0.5 * log((double)tlast); // + 0.5 * log((double)ni); 
	s.bf = logbf; 

/////////////////////////////////////////////////////////////////
	//sample beta;
	double bb= (yy-bob) / 2.0; 
	double tau = gsl_ran_gamma_mt(gsl_r, ni/2.0,  1.0/bb); 

	gsl_vector * vnorm = gsl_vector_alloc(col); 
	for (int m = 0; m < col; m++)
	{
		double tnormal = gsl_ran_gaussian(gsl_r, 1); 
		gsl_vector_set(vnorm, m, tnormal); 
	}   // std normal vector; 
	
	gsl_blas_dtrsv(CblasUpper, CblasNoTrans, CblasNonUnit, ginvOmega, vnorm); 
	//it compuates inv(ginvOmega) %*% vnorm;  
	gsl_vector_scale(vnorm, 1.0/sqrt(tau)); 
    gsl_vector_add(vnorm, res); 
	gsl_vector_scale(vnorm, sqrt(tau)); 
	//now the vnorm contains beta; 
	
	s.beta.clear(); 
	for (int i = 0; i < col; i++)
		s.beta.push_back(gsl_vector_get(vnorm, i)); 
	//this beta is normalized by tau; so that tau = 1; 
    
	gsl_matrix_free(ginvOmega);
	gsl_vector_free(vnorm);
	gsl_vector_free(res);
	gsl_vector_free(gph);
	gsl_vector_free(gyx);


	double loglike = 0;
	double ss = (double) (s.sin.size()); 
	loglike += (ss - 1.0) * log(s.p) + (nLoci - ss) * log(1.0 - s.p); 
	// the above calc prior for the model. 
	s.like = loglike; 
}

void ModelnData::rao_blackwell(class cMCMC& s, double * bforpr, gsl_vector * curZ)
{
	int ni = nCohort; 
	int ns = s.sin.size(); 
	int col = ns; 
	if(ns == 0) return; 
	
	double suma = 0; 
	map<string, real> :: iterator iter; 
	if(m_not_snp == 1) 
		suma = (double) s.sin.size(); 
	else 
	{
		for (unsigned i = 0; i < s.sin.size(); i++)
		{
			string rs;
			rs.assign(vsRsnum.at(s.sin.at(i))); 
			iter = mapRs2var.find(rs); 
			if(iter == mapRs2var.end())
			{
				cout << "-mcmc: snp has no maf: " << rs << endl; 
				safe_exit(); 
			}
			real var = iter->second; 
			suma +=  var; 
		}
	}
	if(suma < 1e-100) suma = 1e-100;  
	s.sigma = sqrt(s.h / (1.0 - s.h) / suma); 
//    s.sigma = FIXED_SIGMA;//grant:fixed_parameters; 

	double inv_va = 1./(s.sigma * s.sigma); 

	gsl_matrix * ginvOmega = gsl_matrix_alloc(col, col); 
	get_gXX(s, s, ginvOmega); 
	for (int i = 0; i < col; i++)
		gsl_matrix_set(ginvOmega, i, i, gsl_matrix_get(ginvOmega, i,i) + inv_va); 

	gsl_linalg_cholesky_decomp(ginvOmega); 
	double logdet = 0; 
	for (int i = 0; i < col; i++)
		logdet += log(fabs(gsl_matrix_get(ginvOmega, i, i))); 
	logdet *= 2.0; 
	
	gsl_vector * gph = gsl_vector_alloc(ni); 
	gsl_vector_memcpy(gph, curZ); 
	if(cc == 1) 
		center(ni, gph); 
	
	gsl_vector * gyx = gsl_vector_calloc(col);
	for (int j = 0; j < col; j++)
	{
		int m = s.sin.at(j); 
		double temp = 0; 
		if(cc == 1) 
		{
			for (int i = 0; i < ni; i++)
				temp += snpmgt[i][m] * gsl_vector_get(gph, i); 
		}
		else 
			temp = m_xty[m]; 
		gsl_vector_set(gyx, j, temp); 
	}   //XtY; 
	
	gsl_vector * res = gsl_vector_alloc(col); 
	gsl_linalg_cholesky_solve(ginvOmega, gyx, res); //res = omega %*% gyx; 
	
   	double bob = 0;
	gsl_blas_ddot(res, gyx, &bob); 
	
	double yy = 0; 
	gsl_blas_ddot(gph, gph, &yy); 
/////////////////////////////////////////////////////////////////////////////////////////	
	
	//sampling;
	double bb= (yy-bob) / 2.0; 
	double tau = gsl_ran_gamma_mt(gsl_r, ni/2.0,  1.0/bb); 
	gsl_vector * vnorm = gsl_vector_alloc(col); 
	for (int j = 0; j < col; j++)
		gsl_vector_set(vnorm, j, gsl_ran_gaussian(gsl_r, 1)); 
	// std normal vector; 
	
	gsl_blas_dtrsv(CblasUpper, CblasNoTrans, CblasNonUnit, ginvOmega, vnorm); 
	gsl_vector_scale(vnorm, 1.0/sqrt(tau)); 
	//it compuates inv(ginvOmega) %*% vnorm;  
	gsl_vector_add(vnorm, res); 
	//now the vnorm contains beta; 
	
/////////////////////////////////////////////////////////////////////////////////////////	
	double * yres = new double[ni]; 
	for (int i = 0; i < ni; i++)
	{
		yres[i] = gsl_vector_get(gph, i) ; 
	 	for (int j = 0; j < col; j++)
	   		yres[i] -=  gsl_vector_get(vnorm, j) * snpmgt[i][s.sin.at(j)]; 
	}
	
	{	
		double tsuma = 0; 
		if(m_not_snp == 1) 
			tsuma = (double) s.sin.size(); 
		else 
		{
			for (unsigned i = 0; i < s.sin.size(); i++)
			{
				int m = s.sin.at(i); 
				string rs(vsRsnum.at(m)); 
				map<string, real> :: iterator iter; 
				iter = mapRs2var.find(rs); 
				
				if(iter == mapRs2var.end())
				{
					cout << "-mcmc: snp has no maf: " << rs << endl; 
					safe_exit(); 
				}
				double var = (double) (iter->second); 
				tsuma += var; 
			}
		}
		if(tsuma < 1e-100) tsuma = 1e-100; 
        
		double sumbeta2 = 0; 
		for (unsigned i = 0; i < s.sin.size(); i++)
		{
			double temp = gsl_vector_get(vnorm, i); 
			sumbeta2 += temp * temp;
		}
		
		real * snpgt = new real[nCohort]; 
	    real * snpph = new real[nCohort]; 
		
		vector<char> snpin; 
		snpin.insert(snpin.begin(), nLoci, '0'); 
		for (unsigned m = 0; m < s.sin.size(); m++)
			snpin.at(s.sin.at(m)) = '1'; 
		
		
		for (int m = 0; m < nLoci; m++)
		{
			if(snpin.at(m) == '1') 
				continue; 
		 
			double var = 0; 
			if(m_not_snp == 1)
				var = 1;
			else 
			{
				string rs(vsRsnum.at(m)); 
				map<string, real> :: iterator iter; 
				iter = mapRs2var.find(rs); 
				if(iter == mapRs2var.end())
				{
					cout << "-mcmc: snp has no maf: " << rs << endl; 
					safe_exit(); 
				}
				var = (double) (iter->second); 
			}
			double suma = tsuma + var; 
			double ratio = s.h / (1.0 - s.h); 
			double sg2_j1 = ratio / suma / tau; 
            double sg2_j0 = ratio / tsuma / tau; 
			
			real sigma = sqrt(sg2_j1 * tau); 

			for (int i = 0; i < ni; i++)
			{
				snpgt[i] = (real) snpmgt[i][m];
				snpph[i] = (real) yres[i];
			}                                 
			//here snpph is centered already; 

			double beta = 0; 
			bforpr[m] = calc_residual_logbf(sigma, snpph, snpgt, tau, ni, beta);
//			bforpr[m] = calc_bf(sigma, snpph, snpgt, ni, beta);

			double logodds = bforpr[m] + log(s.p) - log(1.0 - s.p); 
			logodds -= sumbeta2 * 0.5 * ( 1.0 / sg2_j1 - 1.0 / sg2_j0); 
			logodds -= (ns) * 0.5 * (log(sg2_j1) - log(sg2_j0)); 
			if(logodds > 100) bforpr[nLoci+m] = 1; 
			else if (logodds < -100) bforpr[nLoci+m] = 0; 
			else 
			{
				double odds = exp(logodds); 
				bforpr[nLoci+m] = odds / (1.0 + odds); 
			}
			bforpr[m] = beta * bforpr[nLoci+m]; 
		}   // snps out of the current model; 

		for (unsigned j = 0; j < s.sin.size(); j++)
		{
			double tbeta = gsl_vector_get(vnorm, j);
			double sumbeta2_in = sumbeta2 - (tbeta * tbeta); 
			int m = s.sin.at(j); 
			for (int i = 0; i < ni; i++)
			{
				snpgt[i] = (real) snpmgt[i][m]; 
				snpph[i] = (real) yres[i] + snpgt[i] * gsl_vector_get(vnorm, j); 
			}                            
			//here snpph is centered already; 
			
			double var = 0; 
			if (m_not_snp == 1)
				var = 1.0; 
			else 
			{
				string rs;
				rs.assign(vsRsnum.at(m)); 
				map<string, real> :: iterator iter; 
				iter = mapRs2var.find(rs); 
				if(iter == mapRs2var.end())
				{
					cout << "-mcmc: snp has no maf: " << rs << endl; 
					safe_exit(); 
				}
				var = (double) (iter->second); 
			}
			double suma = tsuma - var; 
			if(suma < 1e-100) suma = 1e-100;  
			double ratio = s.h / (1.0 - s.h); 
			double sg2_j1 = ratio / tsuma / tau; 
            double sg2_j0 = ratio / suma / tau; 
			
			real sigma = sqrt(sg2_j1 * tau); 
			
			double beta = 0; 
			bforpr[m] = calc_residual_logbf(sigma, snpph, snpgt, tau, ni, beta);

			double logodds = bforpr[m] + log(s.p) - log(1.0 - s.p); 
			logodds -= sumbeta2_in * 0.5 * ( 1.0 / sg2_j1 - 1.0 / sg2_j0); 
			logodds -= (ns-1) * 0.5 * (log(sg2_j1) - log(sg2_j0)); 
			//grant; 
			if(logodds > 100) bforpr[nLoci+m] = 1.0; 
			else if (logodds < -100) bforpr[nLoci+m] = 0.0; 
			else 
			{
				double odds = exp(logodds); 
				bforpr[nLoci+m] = odds / (1.0 + odds); 
			}
			bforpr[m] = beta * bforpr[nLoci+m]; 
			//calc prob snp in the model; rao-blackwellization;  
		}   
		
		delete[] snpph;
		delete[] snpgt; 
	}
	delete[] yres; 
	
	gsl_matrix_free(ginvOmega);
	gsl_vector_free(res);
	gsl_vector_free(vnorm);
	gsl_vector_free(gyx);      
	gsl_vector_free(gph);      
}


double ModelnData::propose_z(gsl_vector * gZold, gsl_vector * gZnew, int repeat)
{
	//exchange the rank; 
    gsl_vector_memcpy(gZnew, gZold); 

	for (int r = 0; r < repeat; r++)
	{
	   int index[2];
	   DistinctIntArray(0, icase.size(), 2, index); 
	   int p1 = icase.at(index[0]); 
	   int p2 = icase.at(index[1]); 
	   double t1 = gsl_vector_get(gZnew, p1); 
	   double t2 = gsl_vector_get(gZnew, p2); 
	   gsl_vector_set(gZnew, p1, t2); 
	   gsl_vector_set(gZnew, p2, t1); 
	}
	   
	for (int r = 0; r < repeat; r++)
	{
	   int index[2];
	   DistinctIntArray(0, ictrl.size(), 2, index); 
	   int p1 = ictrl.at(index[0]); 
	   int p2 = ictrl.at(index[1]); 
	   double t1 = gsl_vector_get(gZnew, p1); 
	   double t2 = gsl_vector_get(gZnew, p2); 
	   gsl_vector_set(gZnew, p1, t2); 
	   gsl_vector_set(gZnew, p2, t1); 
	}
    

	//quantitative 
//	int ni = nCohort; 
//	
//	for (int r = 0; r < repeat; r++)
//	{
//	   int index[2];
//	   DistinctIntArray(0, ni, 2, index); 
////	   fplog << index[0] << " " << index[1] << " : "; 
//
//	   double ph1 = gsl_vector_get(gZnew, index[0]); 
//	   double ph2 = gsl_vector_get(gZnew, index[1]); 
//       double sign1 = (ph1 > 0 ? 1.0 : -1.0); 
//       double sign2 = (ph2 > 0 ? 1.0 : -1.0); 
//	   double len = sqrt(ph1 * ph1 + ph2 * ph2); 
//	   double angle = atan(fabs(ph2 / ph1)); 
//	
//	   {
//		   double halfpi = 3.1415926 / 2.0; 
//		   double theta = (gsl_rng_uniform(gsl_r) - 0.5) * 3.1415926 / 30.0; 
//		   theta += angle; 
//		   if(theta > halfpi) theta -= halfpi; 
//		   else if (theta < 0) theta = -theta; 
//		   ph1 = cos(theta) * len * sign1; 
//		   ph2 = sin(theta) * len * sign2; 
////		   double newlen = sqrt(ph1 * ph1 + ph2 * ph2); 
////		   double ratio = len / newlen; 
////		   ratio *= ratio; 
////		   ph1 *= ratio; 
////		   ph2 *= ratio; 
//		   gsl_vector_set(gZnew, index[0], ph1); 
//		   gsl_vector_set(gZnew, index[1], ph2); 
//	   }
//	}
//
//	normalize(ni, gZnew); 
//	//normalize gZnew; 

	return 0.0; 
}

bool poscomp(int a, int b)
{
	return (a < b); 
}

double ModelnData::propose_snp(double rp, int switchrank, vector<int>& in)
{                                                                              
    double loghastings = 0.0; 
	
	map<int, int> :: iterator iter; 
	
	map<int, int> mapi2in; 
	for (unsigned i = 0; i < in.size(); i++)
		mapi2in[in.at(i)] = 1; 

	int nsnpin = (int) in.size();  
	int nsnpout = nLoci - nsnpin; 
	               
	double unif = gsl_rng_uniform(gsl_r); 
	if(unif < 0.40)  //add a snp;  
	{
		if(nsnpin < m_maxsnp && nsnpout > 0)
		{
			int snpkey = -1; 
			do {
				int pick = 0; 
				pick = gsl_ran_discrete(gsl_r, m_discrete); 
				snpkey = m_rank2pos[pick]; 
				iter = mapi2in.find(snpkey);
			} while(iter != mapi2in.end()); 
			
			int rank_of_pick = m_pos2rank[snpkey]; 
			double total = 1.0; 
			for (unsigned i = 0; i < in.size(); i++)
				total -= m_mix_prop[m_pos2rank[in.at(i)]]; 
			double prob_of_pick = m_mix_prop[rank_of_pick] / total; 
			loghastings = - log(prob_of_pick) - log(nsnpin + 1.0); 
			in.push_back(snpkey);                     
		}
		//use mixture of hastings ratio. 
		
//		if(nsnpin < m_maxsnp && out.size() > 0)
//		{
//			int pick = gsl_rng_uniform_int(gsl_r, out.size()); 
//			int snpkey = out.at(pick); 
//			newsnp[snpkey] = '1'; 
//			loghastings = log(out.size() + 0.0) - log(in.size() + 1.0);  
//		}
		
	}
	else if (unif < 0.8)  //remove a snp;
	{
//		if(in.size() > 0 && nsnpin > m_minsnp)
//		{
//			int pick = gsl_rng_uniform_int(gsl_r, in.size()); 
//			int snpkey = in.at(pick); 
//			newsnp[snpkey] = '0'; 
//            loghastings = log(in.size() + 0.0) - log(out.size() + 1.0); 
//		}
		if(nsnpin > m_minsnp)
		{
			int pick = gsl_rng_uniform_int(gsl_r, in.size()); 
			int snpkey = in.at(pick); 
			in.erase(in.begin() + pick); 
			
			int rank_of_pick = m_pos2rank[snpkey]; 
			double total = 1.0; 
			for (unsigned i = 0; i < in.size(); i++)
				total -= m_mix_prop[m_pos2rank[in.at(i)]]; 
			double prob_of_pick = m_mix_prop[rank_of_pick] / total; 
			
			loghastings = log((double) nsnpin) + log(prob_of_pick); 
		}
	}
	else   //switch   
	{
		if(nsnpin > 0 && nsnpout > 0)
		{
			int nbrsize = 10; 
			if(nbrsize >= nLoci)  nbrsize = nLoci - 1; 
			int pin, pout; 
			int pickin = gsl_rng_uniform_int(gsl_r, in.size()); 
			pin = in.at(pickin); 
			map<int, int> :: iterator iter;
			vector<int> candidate; 

			for (int i = -nbrsize; i <= nbrsize; i++)
			{
				int tt = (pin + i + nLoci) % nLoci; 
				iter = mapi2in.find(tt); 
				if(iter == mapi2in.end())
					candidate.push_back(tt); 
			}

			double ncandidate0 = (double) candidate.size(); 
			if(ncandidate0 < 1.0) return 0.0; 
			pout = candidate.at(gsl_rng_uniform_int(gsl_r, candidate.size())); 

			double ncandidate1 = 0.0; 
			for (int i = -nbrsize; i <= nbrsize; i++)
			{
				int tt = (pout + i + nLoci) % nLoci; 
				iter = mapi2in.find(tt); 
				if(iter == mapi2in.end())
					ncandidate1 += 1.0; 
			}

			in.erase(in.begin() + pickin); 
			in.push_back(pout); 
			loghastings += log(ncandidate1 / ncandidate0); 
		}
		 
	}
	stable_sort(in.begin(), in.end(), poscomp); 
	mapi2in.clear(); 
	return loghastings; 
}

double ModelnData::propose(class cMCMC& s, int repeat)
{
	double loghastings = 0.0; 
	for (int i = 0; i < repeat; i++)
	{
		real newh = s.h + (gsl_rng_uniform(gsl_r) - 0.5) * 0.1;
		if (newh > m_maxh) newh = m_maxh*2 - newh; 
		else if (newh < m_minh) newh = m_minh*2 - newh;
//		loghastings += 0.9 * (-log(newh) + log(s.h)); 
		//grant:whs h~beta(0.1, 1) instead of beta(1,1). 
		s.h = newh; 

	}
	
	for (int r = 0; r < repeat; r++)
	{
		double oldp = log10(s.p); 
		double newp = oldp + (gsl_rng_uniform(gsl_r) - 0.5) * 0.05;
		if (newp > m_maxlogp) newp = m_maxlogp*2 - newp; 
		else if (newp < m_minlogp) newp = m_minlogp*2 - newp;
		s.p = pow(10., newp); 
		loghastings += (newp-oldp) * log(10.0); 
	}
	
//	double nin = 1 + s.sin.size();  
//	double nout = 1 + nLoci - s.sin.size(); 
//	double pr_old = gsl_ran_beta_pdf(s.p, nin, nout); 
//	double prop_in_range_old = gsl_cdf_beta_P(pow(10.0, m_maxlogp), nin, nout); 
//	prop_in_range_old -= gsl_cdf_beta_P(pow(10.0, m_minlogp), nin, nout); 
//	pr_old /= prop_in_range_old;    //density of the pr_old; 

	for (int r = 0; r < repeat; r++)   //r < repeat; 
	{       
		double ts = propose_snp(0.7, 0, s.sin); 
		//70% use rank_based proposal, 30% use uniform; 
		loghastings += ts;
	}

//	struct timeval beg, end;
//	struct timezone tz;
//	int getb, gete;
//	getb = gettimeofday(&beg, &tz);
//
//	nin = 1 + s.sin.size(); 
//	nout = 1 + nLoci - s.sin.size(); 
//	double pnew = gsl_ran_beta(gsl_r, nin, nout); 
//	double log10pnew = log10(pnew); 
//	int inrange = 0; 
//	int count = 0; 
//	while(inrange == 0)
//	{   
//		count ++; 
//		pnew = gsl_ran_beta(gsl_r, nin, nout); 
//		log10pnew = log10(pnew);    
//		if((log10pnew > m_maxlogp) || (log10pnew < m_minlogp))
//			inrange = 1; 
//	}   //rejection method; 
//	fplog << count << endl; 
//
//	if(inrange == 0) 
//		pnew = s.p; 
//
//	double pr_new = gsl_ran_beta_pdf(pnew, nin, nout); 
//	double prop_in_range_new = gsl_cdf_beta_P(pow(10.0, m_maxlogp), nin, nout); 
//	prop_in_range_new -= gsl_cdf_beta_P(pow(10.0, m_minlogp), nin, nout); 
//	pr_new /= prop_in_range_new;     //density of the pr_new; 
//	
//	gete = gettimeofday(&end, &tz);
//	if(getb == 0 && gete == 0) 
//		m_clock4 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
//
//	loghastings += log(pr_old) - log(pr_new); 
//	s.p = pnew; 

	return loghastings; 
}


void ModelnData::mcmc()
{
	if (nLoci < (int) m_maxsnp)  m_maxsnp = nLoci; 
	if(cc) m_minsnp = 1; 
	
	m_minlogp = log10(m_pmin) -log10((double) (nLoci));
	m_maxlogp = m_minlogp + min(log10((double)m_pmax), log10((double) nLoci));          //1:1000 out of whatever; 

//	m_maxsnp = 20; 
	//grant:whs
//	m_minlogp = log10(0.5) - log10((double) nLoci); 
//	m_maxlogp = log10(5.0) - log10((double) nLoci);  //(0.5 ~ 5) out of ~500 total SNPs. 
    //grant:whs


	//remove the individuals whose phenotype is "NA"; 
	{
		nCohort = (int) vv_phdex.at(0).size(); 
		for (int i = 0; i < nCohort; i++)
		{
			int j = vv_phdex.at(0).at(i); 
			for (int m = 0; m < nLoci; m++)
				snpmgt[i][m] = snpmgt[j][m]; 
		}
		
		m_ph = new double[nCohort]; 
		for (int i = 0; i < nCohort; i++)
		{
			int j = vv_phdex.at(0).at(i); 
			m_ph[i] = vv_phval.at(0).at(j); 
		}
		//grant; 
	}
	fplog << "### number of individual that has valid phenotypes = " << nCohort << endl; 
	
	for (int m = 0; m < nLoci; m++)
	{
		double sum = 0; 
		int count = 0; 
		double sum2 = 0; 
		for (int i = 0; i < nCohort; i++)
		{
			double mgt = snpmgt[i][m]; 
			if (fabs(mgt - NA) > 1e-6)
			{
				count++; 
				sum += mgt;    
				sum2 += mgt * mgt; 
			}
		}
		sum /= (double) count; 
		sum2 /= (double) count; 
//		double std = sqrt(sum2 - sum * sum); 
		for (int i = 0; i < nCohort; i++)
		{
			double mgt = snpmgt[i][m]; 
			if (fabs(mgt-NA) > 1e-6)
				mgt -= sum; 
			else 
				mgt = 0;      //set missing data to mean; 
			snpmgt[i][m] = mgt; 
		}
	}

//	for (int m = 0; m < nLoci; m++)
//	{
//		for (int i = 0; i < nCohort; i++)
//		{
//			char buf[100]; 
//			sprintf(buf, "%5.3f ", snpmgt[i][m]); 
//			fplog << buf; 
//		}
//		fplog << endl; 
//	}
	
	double * bf1 = new double[nLoci]; 
	m_pos2rank = new long[nLoci]; 
	m_rank2pos = new long[nLoci]; 
	//the following block do single-SNP bf calculations, then break snps into 3 classes using 10%,30% percentile
	{            
		char str[100] = "-mcmc: single snp bf of typed genotypes: "; 
		
		int prg_cur = 0; 
		int prg_all = nLoci; 
		int prg_mod = (int) (nLoci / 100.0); 
		if (prg_mod == 0) prg_mod = 1;
	                                   
		real * snpph = new real[nCohort]; 
		real * snpgt = new real[nCohort]; 

		int ni = nCohort; 
		
		//write mcmc.snpdata.txt 
		fstream outfile; 
		string sfn("output/");
		sfn.append(fnOutput);
		sfn.append(".snp.txt");
		outfile.open(sfn.c_str(), ios::out);
		if(outfile.is_open()) 
		{
			outfile << "## af is the allele freq for A" << endl;  
			outfile << "rs\t A\t B\t af \t var \t chr \t pos \t loc \t bf \t beta" << endl;  
			double beta = 0; 
			for (int m = 0; m < nLoci; m++)
			{
				for (int i = 0; i < nCohort; i++)
				{
					snpgt[i] = (real) snpmgt[i][m];
					snpph[i] = (real) m_ph[i];
				}
//				center(nCohort, snpgt); 
				center(nCohort, snpph); 

				bf1[m] = calc_bf(0, snpph, snpgt, ni, beta) / log(10.0);
				//for casectrl, treat them as quantitative to compuate single snp bfs. 
				string rs = vsRsnum.at(m); 
				char buf[100]; 
				sprintf(buf, "%-s ", rs.c_str()); 
				outfile << buf; 
				if (mapRs2mm[rs].second == 'N') 
					outfile << " N \t" ;
				else 
					outfile << mapRs2mm[rs].second << "\t";
				if (mapRs2mm[rs].first == 'N') 
					outfile << " N \t";
				else 
					outfile << mapRs2mm[rs].first << "\t";
				
				sprintf(buf, "%.3f\t", mapRs2maf[rs]); 
				outfile << buf;
				sprintf(buf, "%.3f\t", mapRs2var[rs]); 
				outfile << buf;
				if(mapRs2chr[rs] < 0)
					outfile << " NA \t"; 
				else
					outfile << mapRs2chr[rs] << "\t"; 
				outfile << mapRs2pos[rs] << "\t" << m << "\t"; 
				sprintf(buf, "%+5.3f ", bf1[m]);
				outfile << buf << beta << endl; 
				prg_cur++; 
				if(prg_cur % prg_mod == 0) print_progress_bar(0, str, prg_cur, prg_all); 
			}                    
			print_progress_bar(1, str, 1, 1); 

			outfile.close(); 
		}
		else 
			cout << "-pimass: mcmc skip writing snpdata" << endl;

		delete[] snpph;
		delete[] snpgt; 
		//sort the bf 
		vector< pair<real, int> > bfsort;
		for (int m = 0; m < nLoci; m++)
		{
			pair<real, int> tp; 
			tp.first = bf1[m];
			tp.second = m;
			bfsort.push_back(tp); 
		}
		stable_sort(bfsort.begin(), bfsort.end(), bfcomp); 

		for (int m = 0; m < nLoci; m++)
			m_pos2rank[m] = bfsort.at(m).second; 
		for (int m = 0; m < nLoci; m++)
			m_rank2pos[m_pos2rank[m]] = m;  
		delete[] bf1; 
		vector< pair<real, int> > ().swap(bfsort);
		
		////////////////////////////////////////////////
		// hash table for xtx; 
//		{
//			long len = min(10, nLoci); 
//			for (long i = 0; i < len; i++)
//			{
//				long p1 = m_rank2pos[i]; 
//				for (long j = i; j < len; j++)         
//				{
//					long p2 = m_rank2pos[j]; 
//					pair<long, long> key (j, i); 
//					double sum = 0; 
//					for (long k = 0; k < nCohort; k++)
//						sum += snpmgt[k][p1] * snpmgt[k][p2]; 
//						
//					m_mapi2xtx[key] = sum; 
//				}
//			}
//		}
		////////////////////////////////////////////////
		if(cc == 0) 
		{
			m_xty = new double[nLoci]; 
			for (int m = 0; m < nLoci; m++)
			{
				m_xty[m] = 0; 
				for (int i = 0; i < nCohort; i++)
					m_xty[m] += snpmgt[i][m] * m_ph[i]; 
			}
		}
		///////////////////////////////////////////////
	}
	//single SNP bfs. 

	{
		int m_meanrank = 2000; 
		double m_rho = 0.70; //70% exp + 30% unif. 
		m_mix_prop = new double[nLoci]; 
		for (int m = 0; m < nLoci; m++)
		{
			double temp = pgeometric(m, nLoci, m_meanrank); 
			if(temp < -100) 
				temp = 0; 
			else 
				temp = exp(temp); 
//			double temp = 0; 
//			if(m < m_meanrank) temp = 1.0 / m_meanrank; 
//			else temp = 0.0; 
			m_mix_prop[m] = m_rho * temp + (1-m_rho) / nLoci; 
//			fplog << m_mix_prop[m] << " "; 
		}
		m_discrete = gsl_ran_discrete_preproc(nLoci, m_mix_prop);
//		for (int i = 0; i < 100000; i++)
//		{
//			int pick = -1; 
//			pick = gsl_ran_discrete(gsl_r, m_discrete); 
//			fplog << pick << " "; 
//		}
//		fplog << endl; 
	}
	
	/////////////////////////////////////////////////////////////////////////////////
	// init the mcmc state;  
	vector<char> snp_01; 
	snp_01.insert(snp_01.end(), nLoci,'0'); 
	
	vector<int> sin; 
	int tnum = min(m_nstart, (int) nLoci);
	for(int r = 0; r < tnum; r++)
	{   
		int m = m_rank2pos[r]; 
		sin.push_back(m); 
	}
	
	double pp = (sin.size()) / (nLoci+1.0); 
    double newp = log10(pp); 
	if (newp > m_maxlogp) newp = m_maxlogp*2 - newp; 
	else if (newp < m_minlogp) newp = m_minlogp*2 - newp;
	pp = pow(10.0, newp); 

	double hh = gsl_rng_uniform(gsl_r) * (m_maxh - m_minh) + m_minh;
//    double pp = pow(10.,(gsl_rng_uniform(gsl_r) * (m_maxlogp-m_minlogp) + m_minlogp));    
	double qq = 0;
	double ll = 0; 

//    pp = FIXED_PI; //grant:fixed_parameters; 

	
	cMCMC cur(hh, pp, qq, ll, sin);
	sin.clear(); 
    //finish initialization; 
    /////////////////////////////////////////////////////////////////////////////////


	//if continuing from the last run; read from the previou realization file. 
	if(0)
	{
		string fnCurx("output/test.realization.txt"); 
		ifstream infile; 
		streambuf * pbuf;
		char delimit[] = ";, \t";
		infile.open(fnCurx.c_str(), ios::in);
		if(!infile.is_open()) 
		{
			cout << "-pimass: cannot open genotype file " << fnCurx << endl; 
			safe_exit(); 
		} 
		pbuf = infile.rdbuf();
		// open file; 

		string line, tline; 
		line.assign(getline(pbuf)); 
		while (!line.empty())
		{
			tline.assign(line); 
			line.assign(getline(pbuf)); 
		}   
        cout << "line: " << line << endl; 
		cout << "tline " << tline << endl; 

		double hh, pp, qq, ll; 
		hh = pp = qq = ll = 0; 
		int snp = 0; 
   		char *res = strtok((char*) tline.c_str(), delimit); 
   		if(res != NULL) hh = atof(res);
		res = strtok(NULL, delimit); 
		res = strtok(NULL, delimit); 
		if(res != NULL) pp = pow(10.0, atof(res)); 
		for (int i = 0; i < 6; i++)
			res = strtok(NULL, delimit); 
		if(res != NULL) snp = atoi(res); 
		vector<int> sin; 
		for (int i = 0; i < snp; i++)
		{
			res = strtok(NULL, delimit); 
			sin.push_back(atoi(res)); 
		}
		cout << "read param: "; 
        cout << hh << "\t" <<  pp << "\t" << qq << "\t" << ll << "\t" << snp << endl; 
		cMCMC tcur(hh, pp, qq, ll, sin); 
		cur.assign(tcur); 
		sin.clear(); 
		cout << cur.h << "\t" << cur.p << "\t"; 
		for (unsigned i = 0; i < cur.sin.size(); i++)
			cout << cur.sin.at(i) << "\t"; 
		cout << endl; 
	}
	/////////////////////////////////////////////////////////////////////////////////
	
	string sfn; 
	sfn.assign("output/");
	sfn.append(fnOutput);
	sfn.append(".path.txt");
	fstream outpath;
	outpath.open(sfn.c_str(), ios::out);
	if(!outpath.is_open()) 
	{
		cout << "-mcmc: cannot open file to write:" << sfn << endl;
		return;
	}
	outpath << "h \t hh \t p \t snp \t ar \t war \t sigma \t bf \t like \t " << endl; 

	sfn.assign("output/");
	sfn.append(fnOutput);
	sfn.append(".gamma.txt");
	fstream outgamma; 
	outgamma.open(sfn.c_str(), ios::out);
	if(!outgamma.is_open()) 
	{
		cout << "-mcmc: cannot open file to write:" << sfn << endl;
		return;
	}
	outgamma << "snp\t "; 
	for (int s = 0; s < m_maxsnp; s++) 
		outgamma << "s" << s << "\t"; 
	outgamma << endl; 

	//open the realization file and write the header line; 
	
	map<int, real> mpos2betarb; 
	map<int, real> mpos2beta; 
	map<int, real> :: iterator biter; 
	map<int, real> mpos2count; 
	map<int, real> :: iterator iter; 
	map<int, real> mpos2post; 
	map<int, real> :: iterator piter; 
	map<string, real> mgene2count; 
	map<string, real> :: iterator riter; 
    //hash for the records; 
	

	if(m_num == 0) m_num = 100; 
	int rbfreq = m_num * 10; 
	double rbcounts = 0; //(double) nMaxSteps / rbfreq; 
	int nChain = m_chains;
	double ladder = m_tau;    //\tau in the progression, or temperature ladder; 
	
	gsl_vector * curgZ = gsl_vector_alloc(nCohort);  
	gsl_vector * nextgZ = gsl_vector_alloc(nCohort); 


	if(cc == 0) 
	{
		for (int i = 0; i < nCohort; i++)
			gsl_vector_set(curgZ, i, m_ph[i]); 
		center(nCohort, curgZ); 
	}
	else if (cc == 1) 
	{
		center(nCohort, m_ph); 

//		for (int i = 0; i < nCohort; i++)
//		{
//			double temp = gsl_ran_ugaussian_tail(gsl_r, 0); 
//			if(m_ph[i] > 0)
//				gsl_vector_set(curgZ, i, temp); 
//			else 
//				gsl_vector_set(curgZ, i, -temp); 
//		}   //init curgZ; 

		icase.clear(); 
	    ictrl.clear(); 
	    for (int i = 0; i < nCohort; i++)
		{
			if(m_ph[i] > 0) 
				icase.push_back(i); 
			else 
				ictrl.push_back(i); 
		}
//		for (unsigned i = 0; i < icase.size(); i++)
//			cout << icase.at(i) << " "; 
//		cout << endl; 
//		for (unsigned i = 0; i < ictrl.size(); i++)
//			cout << ictrl.at(i) << " "; 
//		cout << endl; 

//		random_shuffle(icase.begin(), icase.end()); 
//		random_shuffle(ictrl.begin(), ictrl.end()); 

		for (unsigned i = 0; i < ictrl.size(); i++)
		{
			double temp = gsl_cdf_gaussian_Pinv((i+1.0) / (nCohort+1.0), 1.0); 
            gsl_vector_set(curgZ, ictrl.at(i), temp); 
//			cout << temp << endl; 
		}
		for (unsigned i = 0; i < icase.size(); i++)
		{
			double temp = gsl_cdf_gaussian_Pinv((i+1.0+ictrl.size()) / (nCohort+1.0), 1.0); 
            gsl_vector_set(curgZ, icase.at(i), temp); 
//			cout << temp << endl; 
		}
	}

	
	calc_post(cur, cur, curgZ); 
	
	int prg_cur = 0;
	if (nWarmSteps < 100) nWarmSteps = 0; 
	int prg_all = (nWarmSteps + nMaxSteps) * nChain; 
	int prg_mod = (int) (prg_all / 100.0); 
	if(prg_mod == 0) prg_mod = 1; 
	char str[100]; 
	sprintf(str, "-pimass: mcmc of %d steps: ", nMaxSteps); 

//	double zaccept = 0.0; 
	double accept = 0.0; 
	double proposed = 0.0; 
	double * bforpr = new double[nLoci * 2]; 

	map<int, vector<class cMCMC> > map_int2buf; //from chain index to empirical distribution; 
	map<int, vector<class cMCMC> >:: iterator i2b_from, i2b_to; 
	for (int i = 1; i < nChain; i++)
		map_int2buf[i] = vector<class cMCMC> (); 
	vector<class cMCMC> buf; 
	double ptime, ctime; 
    ptime = ctime = 0; 
    struct timeval beg, end;
	struct timezone tz;
	int getb, gete;

	for (int nc = nChain-1; nc >= 0; nc--)
	{
		double wild_proposed = 0.0; 
		double wild_accept = 0.0; 
		double heat_cur = pow(ladder, (double) nc);   
//		cout << nc << "\t" << heat_cur << endl; 
		i2b_from = map_int2buf.find(nc+2);  //the immediate hotter empirical distribution; 
		if(i2b_from != map_int2buf.end())
			vector<class cMCMC> ().swap(i2b_from->second); 
		//clean the previous empirical distribution. 
		i2b_from = map_int2buf.find(nc+1);  //the immediate hotter empirical distribution; 
		i2b_to = map_int2buf.find(nc); 
		for (int steps = 0; steps < (nWarmSteps + nMaxSteps); steps++)
		{
			real loghastings = 0.0; 
			cMCMC next; 
			next.assign(cur);  
			int next_known = 0; 
			double whichprop = gsl_rng_uniform(gsl_r); 

			getb = gettimeofday(&beg, &tz);
			if(whichprop < m_sw)
			{  
				wild_proposed ++; 
				if(i2b_from == map_int2buf.end()) //the exploring chain; 
				{
					int repeat = 1+gsl_rng_uniform_int(gsl_r, 20); 
					loghastings = propose(next, repeat);
					if(cc == 1)
						loghastings += propose_z(curgZ, nextgZ, repeat); 
				}
				else 
				{
					int pick = gsl_rng_uniform_int(gsl_r, i2b_from->second.size());
					//pick a random sample from the empirical distribution of an immediate hotter chain; 
					next.assign(i2b_from->second.at(pick)); 
					next_known = 1; 
					double post_cur = (cur.bf) / heat_cur / ladder + cur.like; 
					double post_next = (next.bf) / heat_cur / ladder + next.like; 
					loghastings = post_cur - post_next; 
				}
			}                                       
			else
			{
				loghastings = propose(next, 1); 
				if(cc == 1)
					loghastings += propose_z(curgZ, nextgZ, 1); 
			}
			gete = gettimeofday(&end, &tz);
			if(getb == 0 && gete == 0) 
				ptime += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 

			getb = gettimeofday(&beg, &tz);
			if(next_known == 0)
			{
				if(cc == 0)
					calc_post(cur, next, curgZ);
				else //cc == 1;  
					calc_post(cur, next, nextgZ);
			}
			gete = gettimeofday(&end, &tz);
			if(getb == 0 && gete == 0) 
				ctime += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
			
			double	post_next = (next.bf) / heat_cur + next.like; 
			double	post_cur = (cur.bf) / heat_cur + cur.like; 
			
			double alpha = post_next - post_cur + loghastings; 
			if(alpha > 100.0) alpha = 1; 
			else if(alpha < -100.0) alpha = 0; 
			else alpha = exp(alpha); 
			proposed += 1.0; 
			if(gsl_rng_uniform(gsl_r) < alpha)
			{
				accept += 1.0; 
				if(whichprop < m_sw) wild_accept++; 
				cur.assign(next);                  
				if(cc == 1) 
					gsl_vector_memcpy(curgZ, nextgZ); 

			}
	//		else
	//			fplog << next.post << "\t" << next.sin.size() << "\t" << cur.post << "\t" << loghastings << endl; 
			
//			if(cc == 1) 
//			{
//				double loghastings = 0; 
//				propose_z(curgZ, nextgZ); 
//				next.assign(cur); 
//				calc_post(next, nextgZ);
//				
//				double	post_next = next.bf / heat_cur; 
//				double	post_cur = cur.bf / heat_cur; 
//				double alpha = post_next - post_cur + loghastings; 
//				
//				if(alpha > 100.0) alpha = 1; 
//				else if(alpha < -100.0) alpha = 0; 
//				else alpha = exp(alpha); 
//				if(gsl_rng_uniform(gsl_r) < alpha)
//				{
//					zaccept += 1.0; 
//					gsl_vector_memcpy(curgZ, nextgZ); 
//				}
//			}
			
			if(prg_cur++ % prg_mod == 0) 
				print_progress_bar(0, str, prg_cur, prg_all);

			if(steps < nWarmSteps)
			{
				continue; 
			}   //burn-in, return; 
			
			
			if((steps - nWarmSteps) % m_num == 0) 
			{
//				getb = gettimeofday(&beg, &tz);
				calc_hh(cur); 
//				gete = gettimeofday(&end, &tz);
//				if(getb == 0 && gete == 0) 
//					m_clock3 += (end.tv_sec-beg.tv_sec) + 1e-6 * (end.tv_usec - beg.tv_usec); 
				char buf[1000]; 	
				double ratio = (accept) / (proposed + 1.0); 
//				double zratio = (zaccept) / (proposed + 1.0); 
				double wildratio = (wild_accept) / (wild_proposed + 1.0); 
				if(cc == 0) 
					sprintf(buf, "%5.3f\t%5.3f\t%5.3f\t%d\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t",  \
							cur.h, cur.hhat, log10(cur.p),(int) cur.sin.size(), ratio, wildratio, cur.sigma,cur.bf, cur.like); 
				else 
					sprintf(buf, "%5.3f\t%5.3f\t%5.3f\t%d\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t",  \
							cur.h, cur.hhat, log10(cur.p), (int) cur.sin.size(), ratio, wildratio, cur.sigma,cur.bf, cur.like); 
					
				outpath << buf << endl;   		
				
				sprintf(buf, "%d\t", (int)cur.sin.size()); 
				outgamma << buf; 
				for (unsigned s = 0; s < cur.sin.size(); s++)
					outgamma << cur.sin.at(s) << "\t"; 
				for (int s = cur.sin.size(); s <  m_maxsnp; s++)
					outgamma << "NA "; 
				outgamma << endl; 

//				if(cc == 1  && ((steps - nWarmSteps) % (10*m_num) == 0)) 
//				{
//					for (int i = 0; i < nCohort; i++)
//						fplog << gsl_vector_get(curgZ, i) << " "; 
//					fplog << endl; 
//				}

				//adjust buffer size; 
//				double tr = (0.01 + m_clock3) / (0.01 + m_clock4); 
//				if(tr > 0.6 && m_bufsize < 1e6) { m_bufsize *= 1.05; m_clock3=m_clock4=0.0; fplog << "buf up " << m_bufsize  << endl; } 
//				else if(tr < 0.4 && m_bufsize > 1e3) { m_bufsize /= 1.05; m_clock3=m_clock4=0.0;fplog << "buf down " << m_bufsize << endl; }
				m_bufsize = 5.0 * cur.sin.size() * cur.sin.size(); 
				if(m_bufsize < 1e4) m_bufsize = 1e4; 
			}
			
			if(i2b_to != map_int2buf.end()) 
			{          
				if((steps - nWarmSteps) % m_num == 0)
				{
					(i2b_to->second).push_back(cur); 
				}
			}   //record empirical distribution, return; 
			
			
			if(nc == 0)
			{
				for (unsigned i = 0; i < cur.sin.size(); i++) 
				{
					int pos = cur.sin.at(i); 
					iter = mpos2count.find(pos); 
					if(iter == mpos2count.end())
						mpos2count[pos] = 1.0;
					else 
						iter->second ++; 
					biter = mpos2beta.find(pos); 
					if(cc == 0 || cc == 1) 
					{
						if(biter == mpos2beta.end())
							mpos2beta[pos] = cur.beta.at(i);
						else 
							biter->second += cur.beta.at(i); 
					}
					else
					{
						mpos2beta[-1] += cur.beta.at(0); 
						if(biter == mpos2beta.end())
							mpos2beta[pos] = cur.beta.at(i+1);
						else 
							biter->second += cur.beta.at(i+1); 
					}
						
				}   //count of snps;                                          

				if(cc == 0 && ((steps - nWarmSteps) % rbfreq == 0))
				{
					rbcounts += 1.0; 
					rao_blackwell(cur, bforpr, curgZ);

					for (int m = 0; m < nLoci; m++)
					{
						biter = mpos2betarb.find(m); 
						if(biter == mpos2betarb.end())
							mpos2betarb[m] = bforpr[m];
						else 
							biter->second += bforpr[m]; 
						
						piter = mpos2post.find(m); 
						if(piter == mpos2post.end())
							mpos2post[m] = bforpr[nLoci+m]; 
						else 
							piter->second += bforpr[nLoci+m]; 
					}
				}
			}   //end if (nc == 0);
						
		}   //end for steps;
	}
	
	print_progress_bar(1, str, 1, 1);
	outpath.close(); 
	outgamma.close(); 

	fplog << "### proposal time = " << ptime << endl; 
	fplog << "### computation time = " << ctime << endl; 
	fplog << "### get_gXX time = " << m_clock1 << endl; 
	fplog << "### inner product time = " << m_clock3 << endl; 
	fplog << "### get_gXX inner loop  = " << m_clock4 << endl; 
	fplog << "### map_looking + inner_product = " << m_clock2 << endl; 
	fplog << "### recent map lookup = " << m_count1 << endl; 
	fplog << "### main map lookup = " << m_count2 << endl; 
	fplog << "### main map miss = " << m_count3 << endl; 
	
	
	     
	sfn.assign("output/");
	sfn.append(fnOutput);
	sfn.append(".mcmc.txt");
	fstream outmcmc; 
	outmcmc.open(sfn.c_str(), ios::out);
	if(!outmcmc.is_open()) 
	{
		cout << "-mcmc: cannot open file to write: " << sfn << endl;
		return;
	}
	
	if(cc == 0 )
		outmcmc << "rs \t chr \t pos \t postc \t postrb \t beta \t betarb" << endl; 
	else 
		outmcmc << "rs \t chr \t pos \t postc \t beta" << endl; 
//	if(cc == 1) 
//	{
//		outfile << "mu \t 0 \t 0 \t 1 \t"; 
//		char buf[100]; 
//		sprintf(buf, "%5.3f", mpos2beta[-1] / (nMaxSteps)); 
//		outfile << buf << endl; 
//	}
	for (int i = 0; i < nLoci; i++)
	{
		char buf[100]; 
		sprintf(buf, "%-15s\t%d\t%ld\t", vsRsnum.at(i).data(),mapRs2chr[vsRsnum.at(i)], mapRs2pos[vsRsnum.at(i)]); 
        outmcmc << buf; 
		double tcount = 0; 
		iter = mpos2count.find(i); 
		if(iter != mpos2count.end())
			tcount = (double) (iter->second) / (nMaxSteps); 

		double tbeta = 0.0; 
		biter = mpos2beta.find(i); 
		if(biter != mpos2beta.end())
			tbeta = (double) (biter->second) / (iter->second); 
		
		if(cc == 0)
		{
			piter = mpos2post.find(i); 
			double trb = piter->second / rbcounts ; 
			double tbetarb = 0.0; 
			biter = mpos2betarb.find(i); 
			if(biter != mpos2betarb.end())
				tbetarb = (double) (biter->second) / rbcounts; 
	    	sprintf(buf, " %6.5f \t %6.5f \t %6.5f \t %6.5f", tcount, trb, tbeta, tbetarb); 
		}
		else //cc==1;
	    	sprintf(buf, " %6.5f \t %6.5f", tcount, tbeta); 
			
		outmcmc << buf << endl;;
	}
	outmcmc.close();
	delete[] bforpr; 

	delete[] m_ph; 
	if(cc == 0) delete[] m_xty; 
}

cGENE::cGENE()
{
	;
}

cGENE::~cGENE()
{
	;
}

cGENE::cGENE(string sgn, int schr, long sbeg, long send) 
{
	 gn.assign(sgn); 
	 chr = schr; 
	 beg = sbeg; 
	 end = send; 
}	

void cGENE::assign(class cGENE ct)
{
	gn.assign(ct.get_name()); 
	chr = ct.get_chr(); 
	beg = ct.get_beg(); 
	end = ct.get_end(); 
}

cSNP::cSNP(string srs, int schr, long spos, long sindex, pair<char, char> smm, real smaf)
{
	rs.assign(srs); 
	chr = schr; 
	pos = spos; 
	index = sindex; 
	major = smm.first; 
	minor = smm.second; 
	maf = smaf; 
	flag = 0; 
}                                                                         

double ModelnData::calc_residual_logbf(real sigma_a, real * phenoval, real * genotype, double tau, int ni, double& beta)
{  
	if (ni == 0) 
	{
		beta = 0; 
		return 0; 
	}
	
	{
		double inv_va = 1./(sigma_a * sigma_a); 
		
		double yx = 0;                     
		for (int i = 0; i < ni; i++)
			yx += phenoval[i] * genotype[i]; 
	
		double xx = 0; 
		for (int i = 0; i < ni; i++)
			xx += genotype[i] * genotype[i]; 
		
		double omega = 1./(xx + inv_va); 
		
		beta = yx * omega; 
		
		double logbf = 0.5 * (log(omega)) - log(sigma_a); 
		logbf += 0.5 * tau * (beta * yx); 
		//note: since no mu, so no log(ni) * 0.5; 
		return logbf; 
	}
	
//	int ns = 1;
//	int col = m_df + 1; 
//	
//	double	inv_va = 1./(sigma_a * sigma_a); 
//
//	gsl_vector * gph = gsl_vector_alloc(ni); 
//	for (int i = 0; i < ni; i++)
//		gsl_vector_set(gph, i, phenoval[i]); 
//
//	gsl_matrix * gX = gsl_matrix_alloc(ni, col); 
//	for (int i = 0; i < ni; i++)
//	{
//		gsl_matrix_set(gX, i, 0, 1); 
//		gsl_matrix_set(gX, i, 1, genotype[i]);
//	}
//	
//	double yy = 0; 
//	gsl_blas_ddot(gph, gph, &yy); 
//	
//	gsl_matrix * ginvOmega = gsl_matrix_alloc(col, col); 
//	gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
//	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, gX, 0.0, gXtX); 
//
//	gsl_vector * gyx = gsl_vector_alloc(col); 
//	gsl_vector * res = gsl_vector_alloc(col); 
//	
//	gsl_matrix_memcpy(ginvOmega, gXtX); 
//	for (int i = 0; i < ns; i++)
//	{
//		double tmp = 0; 
//		tmp = gsl_matrix_get(ginvOmega, i+1, i+1) + inv_va; 
//		gsl_matrix_set(ginvOmega, i+1, i+1, tmp);
//	}   // add prior on diagnal. 
//	
//	gsl_linalg_cholesky_decomp(ginvOmega); 
//	double logdet = 0; 
//	for (int i = 0; i < col; i++)
//		logdet += log(fabs(gsl_matrix_get(ginvOmega, i, i))); 
//	logdet *= 2.0; 
//	
//	//y^t X;
//	gsl_blas_dgemv(CblasTrans, 1.0, gX, gph, 0.0, gyx); 
//	gsl_linalg_cholesky_solve(ginvOmega, gyx, res); //res = omega %*% gyx; 
//	beta = gsl_vector_get(res, 1); 
//													  
//	double bob = 0;
//	gsl_blas_ddot(res, gyx, &bob); 
//
//	double ph_2 = gsl_vector_get(gyx, 0);
//	ph_2 *= ph_2; 
//	
//	double logbf = -logdet *0.5 + log(ni) * 0.5 - log(sigma_a) + 0.5*tau*(bob - ph_2/ni); 
//	gsl_matrix_free(gXtX);
//	gsl_matrix_free(ginvOmega);
//	gsl_vector_free(gyx);
//	gsl_vector_free(res);
//	
//	gsl_matrix_free(gX);
//	gsl_vector_free(gph); 
//	return (logbf); 
}

double ModelnData::calc_bf(real sigma_a, real * phenoval, real * genotype, int ni, double& beta)
{  
	m_df = 1; 
	if (ni == 0) 
	{
		beta = 0; 
		return 0.0;  
	}
	int ns = 1;
	int col = m_df + 1; 
	
	if(sigma_a < 1e-6) sigma_a = 0.2; 
	real inv_va = 1./(sigma_a * sigma_a); 
	

	gsl_vector * gph = gsl_vector_alloc(ni); 
	for (int i = 0; i < ni; i++)
		gsl_vector_set(gph, i, phenoval[i]); 

	gsl_matrix * gX = gsl_matrix_alloc(ni, col); 
	for (int i = 0; i < ni; i++)
	{
		gsl_matrix_set(gX, i, 0, 1); 
		gsl_matrix_set(gX, i, 1, genotype[i]);
	}
	
	double yy = 0; 
	gsl_blas_ddot(gph, gph, &yy); 
	
	gsl_matrix * ginvOmega = gsl_matrix_alloc(col, col); 
	gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, gX, 0.0, gXtX); 
//	cout << " gX %*% gX" << endl; 
//	for (int i = 0; i < col; i++)
//	{
//		for (int j = 0; j < col; j++)
//			cout << gsl_matrix_get(ginvOmega, i, j) << "\t"; 
//    	cout << endl; 
//	}

	gsl_matrix_memcpy(ginvOmega, gXtX); 
	for (int i = 0; i < ns; i++)
	{
		double tmp = 0; 
		tmp = gsl_matrix_get(ginvOmega, i+1, i+1) + inv_va; 
		gsl_matrix_set(ginvOmega, i+1, i+1, tmp);
	}   // add prior on diagnal. 
	
	gsl_linalg_cholesky_decomp(ginvOmega); 
	double logdet = 0; 
	for (int i = 0; i < col; i++)
		logdet += log(fabs(gsl_matrix_get(ginvOmega, i, i))); 
	logdet *= 2.0; 
	
	//y^t X;
	gsl_vector * gyx = gsl_vector_alloc(col); 
	gsl_vector * res = gsl_vector_alloc(col); 
	
	gsl_blas_dgemv(CblasTrans, 1.0, gX, gph, 0.0, gyx); 
	gsl_linalg_cholesky_solve(ginvOmega, gyx, res); //res = omega %*% gyx; 
	beta = gsl_vector_get(res, 1); 
	
//		cout << "logdet: " << logdet << endl; 
	
//		cout << "gyx : " ; 
//		for (int j = 0; j < col; j++)
//			cout << gsl_vector_get(gyx, j) << " "; 
//		cout << endl; 
//		cout << "res : "; 
//		for (int j = 0; j < col; j++)
//			cout << gsl_vector_get(res, j) << " "; 
//		cout << endl; 
													  
	double bob = 0;
	gsl_blas_ddot(res, gyx, &bob); 
	
	double ph_2 = gsl_vector_get(gyx, 0);
	ph_2 *= ph_2; 
	
	double tlast = (yy - bob) / (yy - ph_2/ni);
//		cout << "tlast: " << tlast << endl; 
										  
	double logvavdn = ns * log((double) inv_va) + log((double)ni);    
	double logbf = -0.5 * logdet + 0.5 * logvavdn - ni * 0.5 * log((double)tlast); 
//		cout << "lobgf: " << logbf << endl; 

	gsl_matrix_free(gXtX);
	gsl_matrix_free(ginvOmega);
	gsl_vector_free(gyx);
	gsl_vector_free(res);
	
	gsl_matrix_free(gX);
	gsl_vector_free(gph); 
	return (logbf);    	
}


//vector manipulation; 
void ModelnData::center(int n, double * array) 
{
    double sum = 0; 
	for (int i = 0; i < n; i++)
		sum += array[i]; 
	sum /= n; 
	for (int i = 0; i < n; i++)
		array[i] -= sum; 
}

void ModelnData::center(int n, real * array) 
{
    double sum = 0; 
	for (int i = 0; i < n; i++)
		sum += array[i]; 
	sum /= n; 
	for (int i = 0; i < n; i++)
		array[i] -= sum; 
}                        

void ModelnData::center(int n, gsl_vector * vec)
{
	double ex1 = 0; 
	for (int i = 0; i < n; i++)
	{
		double temp = gsl_vector_get(vec, i);  
		ex1 += temp; 
	}
	ex1 /= n; 
	gsl_vector_add_constant(vec, -ex1); 
}


void ModelnData::normalize(int n, double * vec)
{
	double ex1 = 0; 
	double ex2 = 0; 
	for (int i = 0; i < n; i++)
	{
		double temp = vec[i];  
		ex1 += temp; 
		ex2 += temp * temp; 
	}
	ex1 /= n; 
	ex2 /= n; 
	double sd= sqrt(ex2 - ex1 * ex1); 
	for (int i = 0; i < n; i++)
		vec[i] /= sd; 
}

void ModelnData::normalize(int n, real * vec)
{
	double ex1 = 0; 
	double ex2 = 0; 
	for (int i = 0; i < n; i++)
	{
		double temp = vec[i];  
		ex1 += temp; 
		ex2 += temp * temp; 
	}
	ex1 /= n; 
	ex2 /= n; 
	double sd= sqrt(ex2 - ex1 * ex1); 
	for (int i = 0; i < n; i++)
		vec[i] /= sd; 
}

void ModelnData::normalize(int n, gsl_vector * vec)
{
	double ex1 = 0; 
	double ex2 = 0; 
	for (int i = 0; i < n; i++)
	{
		double temp = gsl_vector_get(vec, i);  
		ex1 += temp; 
		ex2 += temp * temp; 
	}
	ex1 /= n; 
	ex2 /= n; 
	double sd= sqrt(ex2 - ex1 * ex1); 
	gsl_vector_scale(vec, 1.0/sd); 
}

